<?php
/**************************************************************************
 * $Id: StudIPortAjaxServer.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


// setup the list of allowed functions
$cmdLst = array(
	'matrix' => array(
		'matrixShowHidePart',
		'matrixGetCell',
		'matrixAllocFile',
		'matrixReleaseFile',
		'matrixRemoveFile',
		'matrixCommentSave',
		'matrixGetCellComment'
	),
	'file' => array(
		'fileSaveDetails',
		'fileReloadDetails',
		'fileSaveViews',
		'fileReloadViews',
		'fileGetPublicViews',
		'fileGetViews',
		'fileGetInfo',
		'fileListDelete'
	),
	'upload' => array(
		'uploadGetProgress'
	),
	'view' => array(
		'viewGetInfo',
		'viewCreate',
		'viewDelete',
		'viewSaveBasics',
		'viewReloadBasics',
		'viewSaveAccess',
		'viewReloadAccess',
		'viewSaveFiles',
		'viewReloadFiles',
		'viewSetViewedState'
	),
	'question' => array(
		'questionCreate',
		'questionDelete',
		'questionSaveBasics',
		'questionReloadBasics',
		'questionSaveRequest',
		'questionReloadRequest'
	),
	'response' => array(
		'responseCreate',
		'responseDelete',
		'responseSetViewedState',
	),
	'user' => array(
		'userSearch'
	)
);

// config
require_once('config/StudIPort.config.php');

// StudIP libs
require_once('lib/classes/Avatar.class.php');

// my required libs
require_once('lib/StudIPortAccess.lib.php');
require_once('config/StudIPort.config.php');
require_once('lib/PluginLink.lib.php');
require_once('lib/StudIPortTools.lib.php');
require_once('lib/StudIPortDB.class.php');
require_once('lib/logging.php');


//// actual script

// invoke Stud.IP core
page_open(array('sess' => 'Seminar_Session', 'auth' => 'Seminar_Default_Auth', 'perm' => 'Seminar_Perm', 'user' => 'Seminar_User'));  // TODO: what is happening here precisely?

// check permissions
if (!$perm->have_perm('autor') && !isset($_SESSION['StudIPortExtAcc']['access_code']))
	die('Keine Berechtigung!');
	
//print_r($GLOBALS['user']);

// FIXME will this lead to a Ajax-invocation problem due to expected JSON result?
if (!array_key_exists('cmd', $_REQUEST)) {
	debuglog(_WARN, $GLOBALS['user']->id, "AJAX: No command specified", $_REQUEST);

	echo "no command";
	die;
}
$cmd = $_REQUEST['cmd'];

debuglog(_TRACE, $GLOBALS['user']->id, "AJAX: Start", $_REQUEST);

// prepare dummy response
$res = array('cmd' => $cmd);


// check, if the desired function exists and may be called
$fcnFound = false;
foreach ($cmdLst as $mod => $fcns) {
	if (in_array($cmd, $fcns)) {
		// load module
		require_once($STUDIPORT_AJAX_BASEDIR ."/". $mod. ".php");
		// check, if function exists
		if (function_exists($cmd)) {
			$fcnFound = true;
			$res = array_merge($res, $cmd());  // simply invoke the requesting function
		}
	}
}

// if the function could not be found, issue an error message
if (! $fcnFound) {
	setErrorMsg($res, "Ungültige oder unbekannte Aktion");

	debuglog(_WARN, $GLOBALS['user']->id, "AJAX: Unknown action", $_REQUEST);
}

// pass-through ajax session id
$res["session"] = $_REQUEST["session"];
	
// prepare output
// Test if script is called in JSON context or not
if (stristr($_SERVER["HTTP_ACCEPT"], "application/json") != null) {
	header('Content-Type: application/json');
}
header("Content-Control: no-cache");
header("Last-Modified: ". date("r"));
header("Expires: ". date("r"));
echo json_encode($res);

debuglog(_TRACE, $GLOBALS['user']->id, "AJAX: End");

?>
