<?php
/**************************************************************************
 * $Id: StudIPortFileUpload.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


require_once('config/StudIPort.config.php');
require_once('lib/StudIPortUploadTools.lib.php');
require_once('lib/StudIPortFileTools.lib.php');
require_once('lib/StudIPortTools.lib.php');
require_once('lib/StudIPortDB.class.php');
require_once('lib/PluginLink.lib.php');
require_once('lib/logging.php');

// user auth
page_open(array(
	'sess' => 'Seminar_Session',
	'auth' => 'Seminar_Default_Auth',
	'perm' => 'Seminar_Perm',
	'user' => 'Seminar_User'));
$perm->check('autor');

// get file data
$file['name']    = $_FILES['FileUpload_File']['name'];
$file['type']    = $_FILES['FileUpload_File']['type'];
$file['size']    = $_FILES['FileUpload_File']['size'];
$file['tmpname'] = $_FILES['FileUpload_File']['tmp_name'];
$file['err']     = $_FILES['FileUpload_File']['error'];

// check error state
if ($file['err'] != UPLOAD_ERR_OK) {
	$err = "Unbekannter Fehler beim Hochladen";
	switch ($file['err']) {
		case UPLOAD_ERR_INI_SIZE:
			 //Value: 1; The uploaded file exceeds the upload_max_filesize directive in php.ini. 
			$err = "Dateigr&ouml;ße &uuml;berschreitet Servermaximum"; break;
		case UPLOAD_ERR_FORM_SIZE:
			 //Value: 2; The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form. 
			$err = "Dateigr&ouml;ße &uuml;berschreitet maximal erlaubten Wert"; break;
		case UPLOAD_ERR_PARTIAL:
			 //Value: 3; The uploaded file was only partially uploaded. 
			$err = "Datei konnte nur teilweise &uuml;bertragen werden"; break;
		case UPLOAD_ERR_NO_FILE:
			 //Value: 4; No file was uploaded. 
			$err = "Keine Datei angegeben"; break;
		case UPLOAD_ERR_NO_TMP_DIR:
 			//Value: 6; Missing a temporary folder. Introduced in PHP 4.3.10 and PHP 5.0.3. 
			$err = "Es existiert kein tempor&auml;res Verzeichnis f&uuml;r das Hochladen"; break;
		case UPLOAD_ERR_CANT_WRITE:
			 //Value: 7; Failed to write file to disk. Introduced in PHP 5.1.0. 
			$err = "Datei konnte nicht auf dem Server gespeichert werden"; break;
		case UPLOAD_ERR_EXTENSION:
			// Value: 8; File upload stopped by extension. Introduced in PHP 5.2.0. 
			$err = "Dateityp nicht akzeptiert"; break;
	}
	uploadError($err);
}

// check file size
if ($file['size'] <= 0) {
	uploadError("Es wurde keine Datei übertragen. Vielleicht war die Datei zu groß.");
}
if ($file['size'] > $STUDIPORT_FILES_MAXSIZE) {
	uploadError("Die Datei ist zu gro&szlig;");
}

// get db connection
$db =& new StudIPortDB();

// user id
$userid = $GLOBALS['user']->id;

// check quota
if (uploadGetCurUsage($userid) + $file['size'] > $STUDIPORT_FILES_QUOTA) {
	uploadError("Zu wenig Speicherplatz vorhanden");
}

// create unique filename
do {
	$intFileName = md5(uniqid('', true)); 
	$intFilePath = $STUDIPORT_DOWNLOAD_DIR ."/". $intFileName;
} while (file_exists($intFilePath));

// move file to correct location: move_uploaded_file(...)
if (! @move_uploaded_file($file['tmpname'], $intFilePath)) {
	uploadError("Fehler beim Anlegen der Datei");
}

// obtain file type from mime fragment before slash (the mime group)
$type_id = null;
/// step 1: try to match exact mimetype in mime-type table
$db->queryf("SELECT type_id FROM studiport_file_mime WHERE mime_type='%s'", $file['type']);
if ($db->num_rows() > 0) {
	$db->next_record();
	$type_id = $db->f("type_id");
}
/// step 2: if step 1 failed, try to get general type from file-type table
if (empty($type_id)) {
	$mimefrag    = strtolower(substr($file['type'], 0, strpos($file['type'], '/')));
	$mimefraglen = strlen($mimefrag);
	if ($mimefraglen > 0) {
		$db->queryf("SELECT type_id FROM studiport_file_types WHERE mimefrag='%s'", $mimefrag);
		if ($db->num_rows() > 0) {
			$db->next_record();
			$type_id = $db->f("type_id");
		}
	}
}
/// step 3: use default type
if (empty($type_id)) {
	$type_id = $STUDIPORT_FILES_DEFAULTTYPE;
}

// we must truncate the original filename, if it exceeds the max capacity of the table
// TODO
$file['name'] = sanitizeInput($file['name']);

// make sure that we don't have duplicate files names
// get file basename and extension
$dotAt = strrpos($file['name'], '.');
if ($dotAt === false) {
	$fbasename = $file['name'];
	$fext      = '';
} else {
	$fbasename = substr($file['name'], 0, $dotAt);
	$fext      = substr($file['name'], $dotAt);
}

// find first none existing version of file in db
/*
$ftestname = $file['name'];
for ($n = 2; $n <= $STUDIPORT_FILES_MAXDUP; $n++) {
	// IMPROVE and CHECK title vs filename
	$db->queryf("SELECT file_id FROM studiport_file WHERE filename='%s' AND user_id='%s'",
		$ftestname, $userid);
	if ($db->num_rows() == 0) {
		break;
	}
	$ftestname = $fbasename ." (". $n .")". $fext;
}
$file['name'] = $ftestname;
*/
$db->queryf(
	"SELECT ".
	"  DISTINCT filename ".
	"FROM studiport_file ".
	"WHERE user_id='%s' ".
	"  AND filename LIKE '%s%%%s' ".
	"ORDER BY filename DESC ",
	$userid, $fbasename, $fext);
if ($db->num_rows() > 0) {
	$n = 0;
	while ($db->next_record()) {
		if (strcasecmp($db->f('filename'), $file['name']) == 0) {
			$n = max(2, $n);
			continue;
		}
		$tmp  = trim(substr($db->f('filename'), strlen($fbasename)));
		$lpos = strpos($tmp, '(');
		if ($lpos == 0) {
			$tmp  = substr($tmp, 1);
			$rpos = strrpos($tmp, ')');
			if ($rpos == strlen($tmp) - strlen($fext) - 1) {
				$tmp = trim(substr($tmp, 0, $rpos));
				if (! strcmp(intval($tmp), $tmp)) {
					$n = max(intval($tmp) + 1, $n);
					break;
				}
			}
		}
	}
	if (true || $n >= 2) {
		$file['name'] = $fbasename ." (". $n .")". $fext;
	}
}

// insert everthing into DB
$db->queryf("INSERT INTO studiport_file SET "
	. "int_filename  = '%s', "
	. "filename      = '%s', "
	. "type_id       = '%d', "
	. "mimetype      = '%s', "
	. "filesize      = '%d', "
	. "created       = NOW(), "
	. "edited        = NOW(), "
	. "title         = '%s', "
	. "descr_int     = '', "
	. "descr_ext     = '', "
	. "user_id       = '%s'",
	$intFileName, $file['name'], $type_id, $file['type'], $file['size'],
	$file['name'], $userid);

// check, if data could be inserted in mysql
if ($db->affected_rows() == 0) {
	@unlink($intFilePath);
	uploadError('filUpload', "Datei konnte nicht in Datenbank eingef&uuml;gt werden");
}

// get new list of unassociated files
$fileObj = filesGetNoAssocList();
$fileList = array(
	'num' => $fileObj['number'],
	'htmlContent' => utf8_encode($fileObj['htmlContent'])
);
$fileListJSON = json_encode($fileList);

// get new quota statistics
$quota = uploadGetUsageStat($userid);
foreach ($quota as $k=>$v) {
	$quota[$k] = utf8_encode($v);
}
$quotaJSON = json_encode($quota);

// set success message and create a new upload key
$msg  = "Die Datei '". $file['name'] ."' wurde erfolgreich hochgeladen und kann nun zugeordnet werden.";
$pkey = createUploadKey();

// show success or error message
// NOTE reloading Prototype is required by Konqueror (and possibly Safari) to
// make things like update() work
echo "<html><head>\n";
echo "<script type='text/javascript' src='". Assets::url('javascripts/prototype.js') ."'></script>\n";
echo "<script type='text/javascript'>\n";
echo "var pkey    = '". $pkey ."'\n";
echo "var msg     = '". addslashes(htmlentities($msg)) ."'\n";
echo "var fileLst = ". $fileListJSON ."\n";
echo "var quota   = ". $quotaJSON ."\n";
echo "top.fileUpload.success(pkey, msg, quota, fileLst);\n";
echo "</script>\n";
echo "</head><body></body></html>\n";

// end of script, functions below
function uploadError($msg) {
	$pkey = createUploadKey();
	// NOTE reloading Prototype is required by Konqueror (and possibly Safari) to
	// make things like update() work
	echo "<html><head>\n";
	echo "<script type='text/javascript' src='". Assets::url('javascripts/prototype.js') ."'></script>\n";
	echo "<script type='text/javascript'>\ntop.fileUpload.error('$pkey', '". addslashes(htmlentities($msg)) ."');\n</script>\n";
	echo "</head><body></body></html>\n";
	exit;
}

?>
