<?php
/**************************************************************************
 * $Id: StudIPortSystemPlugin.class.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/

 
// include some libs
require_once('config/StudIPort.config.php');
require_once('lib/logging.php');
require_once('lib/PluginLink.lib.php');
require_once('lib/StudIPortDB.class.php');
require_once('lib/StudIPortAccess.lib.php');

// include some studip libs
require_once('vendor/flexi/flexi.php');
require_once('lib/include/reiter.inc.php');

/** @class StudIPortSystemPlugin
 * @extends AbstractStudIPSystemPlugin
 * @brief Pluginclass, called by studip
 */
class StudIPortSystemPlugin extends AbstractStudIPSystemPlugin
{
	protected  $factory;	// a template factory

	// the navigation structure
	private $tabs = array(
		"overview"    => array("dispName" => "bersicht",          "inCat" => ""),
		"news"        => array("dispName" => "Neuigkeiten",        "inCat" => "overview"),
		"myportfolio" => array("dispName" => "Mein ePortfolio",    "inCat" => ""),
		"matrix"      => array("dispName" => "Matrix",             "inCat" => "myportfolio"),
		"artefacts"   => array("dispName" => "Dateien",            "inCat" => "myportfolio"),
		"views"       => array("dispName" => "Ansichten",          "inCat" => "myportfolio"),
		"others"      => array("dispName" => "Andere ePortfolios", "inCat" => ""),
		"help"        => array("dispName" => "Hilfe",              "inCat" => ""),
		"contact"     => array("dispName" => "Kontakt",            "inCat" => "help"),
		"version"     => array("dispName" => "Versionsinfos",      "inCat" => "help")
	);

	/**
	 * @param void
	 */
	public function __construct () {
		parent::AbstractStudIPSystemPlugin();

		// initialize navigation
		$nav =& new PluginNavigation(_("ePortfolio"));
		$this->setNavigation($nav);
		
		// set our icon depending on userHasNews()
		global $STUDIPORT_IMG_FOLDER;
		$this->setPluginiconname("/$STUDIPORT_IMG_FOLDER/header_studiport". ($this->userHasNews() ? "_red" : "") .".png");
	}

	/**
	 * @param void
	 */
	public function __destruct () {
		debuglog(_TRACE, $this->getUser()->getUserID(), "END: PLUGIN");
	}

	/** is executed when a pluginpage is shown
	 * inserts additionally required CSS and JavaScript
	 * @param void
	 */
	public function initialize() {
		debuglog(_TRACE, $this->getUser()->getUserID(), "START: PLUGIN");

		global $HELP_KEYWORD, $EXTERNAL_HELP_LOCATIONID;
		$HELP_KEYWORD="Basis.ePortfolio";
		$EXTERNAL_HELP_LOCATIONID = tuhh;

		// FIXME localization and foreign language support ....
		setlocale(LC_ALL, 'de_DE');
		new StudIPortDB("SET lc_time_names = 'de_DE'");

		// set additional headers
		global $_include_additional_header;
		$url = $this->getPluginURL();

		// get global uri constants
		global $STUDIPORT_URI;
		global $STUDIPORT_IMG_URI;
		global $STUDIPORT_CSS_URI;
		global $STUDIPORT_JS_URI;

		// include css
		$_include_additional_header .=
			"<link rel='stylesheet' href='". $STUDIPORT_CSS_URI ."/StudIPort.css' type='text/css' />\n" .
			"<link rel='stylesheet' href='". $STUDIPORT_CSS_URI ."/Calendar.css' type='text/css' />\n" .
			"<link rel='stylesheet' href='". $STUDIPORT_CSS_URI ."/fakeUploadButton.css' type='text/css' />\n";

		// include js
		$_include_additional_header .=
			"<script type='text/javascript'>var StudIPortImgUri = '". $STUDIPORT_IMG_URI ."/';</script>\n" .
			"<script type='text/javascript'>var StudIPortUrl = '". $STUDIPORT_URI ."/';</script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/CalendarPopup.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/StudIPortDefault.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/matrix.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/cache.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/msgBox.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/upload.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/file.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/ajax.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/view.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/question.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/response.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/fakeUploadButton.js'></script>\n" .
			"<script type='text/javascript' src='". $STUDIPORT_JS_URI ."/box.js'></script>\n";
	}
    
	/** returns the name
	 * @param void
	 */
	public function getPluginname() {
		return _("StudIPort");
	}

	/** shows the homepage
	 * @param void
	 */
	public function actionshow($subnavi) {
		$this->actionoverview($subnavi);
	}

	/**	shows the overview
	 * @param void
	 */
	public function actionoverview($subnavi) {
		// set the root subpage
		if (empty($subnavi)) {
			$subnavi = "news";
		}
		
		$this->show("overview", $subnavi);
	}

	/**	shows my portfolio
	 * @param void
	 */
	public function actionmyportfolio($subnavi) {
		// set the root subpage
		if (empty($subnavi)) {
			$subnavi = "matrix";
		}

		$this->show("myportfolio", $subnavi);
	}

	/**	shows other portfolios
	 * @param void
	 */
	public function actionothers($subnavi) {
		// set the root subpage
		if (empty($subnavi)) {
			$subnavi = "others";
		}

		$this->show("others", $subnavi);
	}

	/**	shows the help
	 * @param void
	 */
	public function actionhelp($subnavi) {
		// set the root subpage
		if (empty($subnavi)) {
			$subnavi = "contact";
		}

		$this->show("help", $subnavi);
	}

	/** generates the tab structure to display
	 * @param active plugin page
	 */
	private function displayTabs($active) {
		$structure = array();

		foreach ($this->tabs as $tab => &$data) {
			if (empty($data["inCat"])) {
		 		$link = PluginEngine::getLink($this, NULL, $tab);
		 	} else {
		 		$link = PluginEngine::getLink($this, array("plugin_subnavi_params" => $tab), $data["inCat"]);
		 	}

			$structure[$tab] = array(
				"topKat" => $data["inCat"],
				"name"   => _($data["dispName"]),
				"link"   => $link,
				"active" => ($tab == $active)
			);
		}

		$reiter = new reiter;
		$reiter->create($structure, $active);
	}

	/** shows the pluginpages
	 * @param action the parent page to show
	 * @param subnavi the subpage to show
	 * TODO perhabps we need some rework here (duplicated code over and over again)
	 */
	public function show($action, $subnavi = NULL) {
		// DEBUG
    	// $memory = memory_get_usage();
	   	// $time = microtime();
	   	
	    // create a template factory
		$this->factory = new Flexi_TemplateFactory(dirname(__FILE__) ."/templates");
		$template = $this->factory->open("empty.php");
		$template->set_layout("StudIPortLayout.php");
		$template->set_attribute("url",  $this->getPluginURL());

		// get the include path
		$incPath_ = dirname(__FILE__) ."/lib/";

		// if subnavi isn't set, take action instead
		$this->displayTabs(empty($subnavi) ? $action : $subnavi);
		
		// DEBUG
		// echo '<pre>';
		// print_r($GLOBALS);
		// echo '</pre>';

		switch($subnavi) {
			case 'artefacts':
				if (empty($_REQUEST['file_id'])) {
					require_once($incPath_ ."StudIPortPageFileListOwner.class.php");
					$files = new StudIPortPageFileListOwner($this->getUser());
					$files->loadData();
					$template->set_attribute('StudIPortContent',
						$files->render($this->factory,
						"StudIPortFileList.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $files->renderInfobox($this->getPluginURL()));
				} else {
					require_once($incPath_ ."StudIPortPageFileDetails.class.php");
					$details = new StudIPortPageFileDetails($this->getUser(), $_REQUEST['file_id']);
					$details->loadData();
					$template->set_attribute('StudIPortContent',
						$details->render($this->factory,
						"StudIPortFileDetails.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $details->renderInfobox($this->getPluginURL()));
				}
				break;

			case 'views':
				if (! empty($_REQUEST['question_id'])) {
					require_once($incPath_ ."StudIPortPageQuestionDetails.class.php");
					$quest = new StudIPortPageQuestionDetails($this->getUser(), $_REQUEST['question_id']);
					$quest->loadData();
					$template->set_attribute('StudIPortContent',
						$quest->render($this->factory,
						"StudIPortQuestionDetails.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $quest->renderInfobox($this->getPluginURL()));
				} else if (! empty($_REQUEST['view_id'])) {
					require_once($incPath_ ."StudIPortPageViewDetails.class.php");
					$view = new StudIPortPageViewDetails($this->getUser(), $_REQUEST['view_id']);
					$view->loadData();
					$template->set_attribute('StudIPortContent',
						$view->render($this->factory,
						"StudIPortViewDetails.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $view->renderInfobox($this->getPluginURL()));
				} else {
					require_once($incPath_ ."StudIPortPageViewList.class.php");
					$view = new StudIPortPageViewList($this->getUser());
					$view->loadData();
					$template->set_attribute('StudIPortContent',
						$view->render($this->factory,
						"StudIPortViewList.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $view->renderInfobox($this->getPluginURL()));
				}
				break;

			case 'matrix':
				require_once($incPath_ ."StudIPortPageMatrix.class.php");
				$matrix = new StudIPortPageMatrix($this->getUser());
				$matrix->loadData();
				$template->set_attribute('StudIPortContent',
					$matrix->render($this->factory,
					"StudIPortMatrix.page.php",
					$this->getPluginURL()));
				$template->set_attribute('infobox', $matrix->renderInfobox($this->getPluginURL()));
				break;

			case 'news':
				require_once($incPath_ ."StudIPortPageOverview.class.php");
				$overview = new StudIPortPageOverview($this->getUser());
				$overview->loadData();
				$template->set_attribute('StudIPortContent',
					$overview->render($this->factory,
					"StudIPortOverview.page.php",
					$this->getPluginURL()));
				$template->set_attribute('infobox', $overview->renderInfobox($this->getPluginURL()));
				break;

			case 'others':
				if (empty($_REQUEST['view_id'])) {
					require_once($incPath_ ."StudIPortPageViewListAlien.class.php");
					$viewlist = new StudIPortPageViewListAlien($this->getUser());
					$viewlist->loadData();
					$template->set_attribute('StudIPortContent',
						$viewlist->render($this->factory, "StudIPortViewListAlien.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $viewlist->renderInfobox($this->getPluginURL()));
				} else {
					require_once($incPath_ ."StudIPortPageViewAlien.class.php");
					$view = new StudIPortPageViewAlien(
						$this->getUser(),
						$_REQUEST['view_id']);
					$view->loadData();
					$template->set_attribute('StudIPortContent',
						$view->render($this->factory, "StudIPortViewAlien.page.php",
						$this->getPluginURL()));
					$template->set_attribute('infobox', $view->renderInfobox($this->getPluginURL()));
				}
				break;

			case 'contact':
				// hier sollte vielleicht noch was hin
				$t = $this->factory->open("StudIPortContact.page.php");
				$t->render();
				$t->set_attribute('url', $this->getPluginURL());
				$template->set_attribute('StudIPortContent', $t->render());
				//$template->set_attribute('StudIPortContent',"Ausf&uuml;hrliche Bugmeldungen per E-Mail an christian.renner@tu-harburg.de");

				break;

			case 'version':
				require_once($incPath_ ."Version.class.php");
				$v = new Version();
				$v->render();
				//$template->set_attribute('StudIPortContent',"StudIPort Plugin<br />Release: ". $GLOBALS['STUDIPORT_RELEASE'] ." &raquo;". $GLOBALS['STUDIPORT_RELEASENAME'] ."&laquo; <br />Version: ". $GLOBALS['STUDIPORT_VERSION']);

				break;
		}

		echo $template->render();

		// DEBUG
        // $time_2 = microtime();
        // $memory_2 = memory_get_usage();
        // echo $time_2 - $time ."<br />";
        // echo $memory_2 - $memory ."<br />";
        // echo memory_get_peak_usage() ." bytes<br />";
	}
    
    /** checks if user has news
     * @param void
     */
    private function userHasNews() {
    
		$sql =& new StudIPortDB();
		
		// TODO remove this (but change implementation of overview.lib before)
		global $studiport_nothing_new;
		$studiport_nothing_new = true;
		
		// any new or edited views?
		$sql->queryf("SELECT view_id " .
				"FROM studiport_view_access access " .
				"LEFT JOIN studiport_view view USING(view_id) " .
				"WHERE access.user_id = '%s' " .
				"AND (NOT view.expires OR CURDATE() <= view.expires) " .
				"AND (viewed = 'N' OR (viewed = 'Y' AND access.accesstime < view.edited))" .
				"LIMIT 0,1", $this->getUser()->getUserId());

		if ($sql->num_rows() != 0) {
			$studiport_nothing_new = false;
        		return true;
        	}

		// new responses?
		$sql->queryf("SELECT response_id " .
        		"FROM studiport_feedback_response r " .
        		"LEFT JOIN studiport_feedback_question USING (question_id) " .
        		"LEFT JOIN studiport_view view USING (view_id) " .
        		"WHERE view.user_id = '%s' " .
        		"  AND viewed != 'Y' " .
        		"  AND DATEDIFF(now(), r.created) BETWEEN 0 AND %d " .
        		"LIMIT 0,1", $this->getUser()->getUserId(), STUDIPORT_DAYS_NEW);



		if ($sql->num_rows() != 0) {
			$studiport_nothing_new = false;
			return true;
		}

		// new questions?
/*
		$sql->queryf("SELECT user_id
				FROM studiport_feedback_request req
				JOIN studiport_feedback_question quest USING (question_id)
				WHERE req.user_id = '%s' AND (NOT quest.expires OR quest.expires > now()) 
				LIMIT 0,1", $this->getUser()->getUserId());

        if ($sql->num_rows() != 0) {
        	$studiport_nothing_new = false;
			return true;
        }
*/
        
		// nothing
        return false;
    }
}
