<?php
/**************************************************************************
 * $Id: file.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


require_once('lib/StudIPortFileTools.lib.php');


/**
 * files
 */

function fileSaveDetails() {
   $res       = array('success' => false);
   $setClause = new MySQLSetClause();

        // file sanity check
   $file_id = $_REQUEST['file_id'];
   if (! validateFileAccess($file_id, $GLOBALS['user']->id, $res)) {
      return $res;
   }
   
   // TODO actually perform more appropriate checks here. We need a config file / or section
   // for all fields providing the needed matches, e.g., minimum and maximum length.
   // An error message must be issued for each failing check. The result is to be displayed
   // in Stud.IP, i.e., either some success message ("data saved") or all errors.

   /*** INPUT VALIDATION ***/
   // title
   $title =& $_REQUEST['title'];
   if (isset($title)) {
      $title = sanitizeInput($title);
      $setClause->set('title', $title);
   }

   // filename
   $filename =& $_REQUEST['filename'];
   if (isset($filename)) {
      $filename = sanitizeInput($filename);
      $setClause->set('filename', $filename);
   }

   // descr_int
   $descr_int =& $_REQUEST['descr_int'];
   if (isset($descr_int)) {
      $descr_int = sanitizeInput($descr_int);
      $setClause->set('descr_int', $descr_int);
   }
   
   // descr_ext
   $descr_ext =& $_REQUEST['descr_ext'];
   if (isset($descr_ext)) {
      $descr_ext = sanitizeInput($descr_ext);
      $setClause->set('descr_ext', $descr_ext);
   }

   // TODO abort, if errors have been encountered
   //if (count($formErrors) > 0) {
   //      setErrorMsg($res, "$updateStr");
   //      return $res;
   //}

   // TODO verify this assumption!
   if ($setClause->count() == 0) {
      $res['success'] = true;  // nothing to change is a success
      return $res;
   }
   
   /*** storage ***/
   $db =& new DB_Seminar();
   $file_id_esc = mysql_real_escape_string($file_id);
   $db->query("UPDATE studiport_file SET ". $setClause->get() ." WHERE file_id='$file_id_esc'");

   // simply reload view data and utf-8 encode (for json)
   $file = fileGetDetails($file_id);
   if ($file != null) {
      array_utf8_encode($file);
   }

   // prepare result
   $res['formUpdates'] = $file;
   $res['success'] = true;
   return $res;
}


function fileReloadDetails() {
   $res = array('success' => false);

   // file sanity check
   if (! validateFileAccess($_REQUEST['file_id'], $GLOBALS['user']->id, $res)) {
      return $res;
   }

   // reload data
   $file = fileGetDetails(intval($_REQUEST['file_id']));
   if ($file != null) {
      array_utf8_encode($file);
   } else {
      setErrorMsg($res, "Datei existiert nicht");
   }

   // prepare result
   $res['formUpdates'] = $file;
   $res['success'] = true;
   return $res;
}


function fileSaveViews() {
   $res = array('success' => false);

   // file sanity check
   $file_id = intval($_REQUEST['file_id']);
   $user_id = $GLOBALS['user']->id;
   if (! validateFileAccess($file_id, $user_id, $res)) {
      return $res;
   }

   // create DB connection and sanitize view id
   $db =& new DB_Seminar();
   $file_id_esc = mysql_real_escape_string($file_id);
   $user_id_esc = mysql_real_escape_string($user_id);

   // convert selected views into an array
   $selViews = empty($_REQUEST['views']) ?
      array() :
      array_filter(array_unique(split(':', $_REQUEST['views'])));

   // get all views belonging to user
   $db->query("SELECT DISTINCT view_id FROM studiport_view WHERE user_id='". $user_id_esc ."'");
   $userViews = array();
   while ($db->next_record()) {
      array_push($userViews, $db->f("view_id"));
   }

   // now verify that all selected view really belong to the user
   foreach ($selViews as $v) {
      if (! in_array($v, $userViews)) {
         setErrorMsg($res, "Ungültige Ansicht");
         return $res;
      }
   }

   // get views file currently belongs to
   $db->query("SELECT view_id FROM studiport_view_files WHERE file_id='$file_id_esc'");
   $oldViews = array();
   while ($db->next_record()) {
      array_push($oldViews, $db->f('view_id'));
   }

   // obtain views that are present in the old and new selection
   $keepViews = array_intersect($oldViews, $selViews);

   // then identify the sets of views that must be added and removed
   $delViews = array_diff($oldViews, $keepViews);
   $addViews = array_diff($selViews, $keepViews);

   // delete obsolete views from assoc table
   if (count($delViews) > 0) {
      $delLst = "'" . join("', '", $delViews) . "'";
      $db->query("DELETE FROM studiport_view_files WHERE file_id='$file_id_esc' AND view_id IN (". $delLst .")");
      if ($db->affected_rows() != count($delViews)) {
         addWarning($res, "Einige Zuordnungen konnten nicht aufgehoben werden");
      }
   }

   // insert new ones
   if (count($addViews) > 0) {
      $addLst = "('". $file_id_esc ."', '". join("'), ('". $file_id_esc. "', '", $addViews) ."')";
      $db->query("INSERT INTO studiport_view_files (file_id, view_id) VALUES ". $addLst);
      if ($db->affected_rows() != count($addFiles)) {
         addWarning($res, "Es konnten nicht alle neuen Zuordnungen durchgeführt werden");
      }
   }

   // if we have changed anything, update view edited timestamp
   /*if (count($delFiles) > 0 || count($addFiles) > 0) {
      $db->query("UPDATE studiport_file SET edited=NOW() WHERE file_id='". $file_id_esc ."'");
   }*/
   
   // return updated list of files
   return fileReloadViews($res);
}


function fileReloadViews() {
   $res = array(success => false);

   // file sanity check
   if (! validateFileAccess($_REQUEST['file_id'], $GLOBALS['user']->id, $res)) {
      return $res;
   }

   // get ID's of views sharing this file
   $db =& new DB_Seminar();
   $file_id_esc = mysql_real_escape_string($_REQUEST['file_id']);
   $db->query("SELECT DISTINCT view_id FROM studiport_view_files WHERE file_id='". $file_id_esc ."'");

   // now turn result set into a list/an array
   $viewList = array();

   while ($db->next_record()) {
      array_push($viewList, $db->f('view_id'));
   }

   // prepare result
   $res['success']  = true;
   $res['viewList'] = $viewList;

   return $res;
}


function fileGetPublicViews() {
   $res = array('success' => false);
   
   $file_id = $_REQUEST['file_id'];
   
   // file sanity check
   if (!validateFileAccess($file_id, $GLOBALS['user']->id, $res)) {
      return $res;
   }
   
   $db =& new DB_Seminar();
   $file_id_esc = intval($file_id);
   
   $db->query("SELECT view_id, name FROM studiport_view LEFT JOIN studiport_view_files USING (view_id) WHERE public='Y' AND file_id='$file_id_esc'");
   
   $htmlContent = '<div class="boxHeader">&Ouml;ffentliche Ansichten</div><ul>';
   
   if ($db->num_rows() > 0) {
      while ($db->next_record()) {
         $link         = getPluginLink('views', array('view_id' => $db->Record['view_id']));
         $htmlContent .= '<li><div class="listTitle"><a href="' . $link . '">' . $db->Record['name'] . '</a></div></li>';
      }
   } else {
      $htmlContent .= '<li><div class="listTitle">Keine Ansichten vorhanden.</div></li>';
   }
   
   $htmlContent .= "</ul>";
   
   $res['success']     = true;
   $res['htmlContent'] = utf8_encode($htmlContent);
   
   return $res;
}


function fileGetViews() {
   $res = array('success' => false);
   
   $file_id = $_REQUEST['file_id'];
   
   // file sanity check
   if (!validateFileAccess($file_id, $GLOBALS['user']->id, $res)) {
      return $res;
   }
   
   $db =& new DB_Seminar();
   $file_id_esc = intval($file_id);
   
   $db->query("SELECT view_id, name FROM studiport_view LEFT JOIN studiport_view_files USING (view_id) WHERE file_id='$file_id_esc'");
   
   $htmlContent = '<div class="boxHeader">Ansichten</div><ul>';
   
   if ($db->num_rows() > 0) {
      while ($db->next_record()) {
         $link         = getPluginLink('views', array('view_id' => $db->Record['view_id']));
         $htmlContent .= '<li><div class="listTitle"><a href="' . $link . '">' . $db->Record['name'] . '</a></div></li>';
      }
   } else {
      $htmlContent .= '<li><div class="listTitle">Keine Ansichten vorhanden.</div></li>';
   }
   
   $htmlContent .= "</ul>";
   
   $res['success']     = true;
   $res['htmlContent'] = utf8_encode($htmlContent);
   
   return $res;
}

function fileGetInfo() {
   $res = array('success' => false);
   
   $file_id = $_REQUEST['file_id'];
   
   // file sanity check
   if (!validateFileAccess($file_id, $GLOBALS['user']->id, $res)) {
      return $res;
   }
   
   $db =& new DB_Seminar();
   $file_id_esc = intval($file_id);
   
   $db->query("SELECT descr_ext FROM studiport_file WHERE file_id='$file_id_esc'");
   
   $htmlContent = '<div class="boxHeader">Beschreibung</div>';
   
   $db->next_record();
   if (strlen($db->Record['descr_ext']) > 0) {
      $htmlContent .= '<div class="boxContent">' . $db->Record['descr_ext'] . '</div>';
   } else {
      $htmlContent .= '<div class="boxContent">Keine Beschreibung vorhanden.</div>';
   }
   
   
   $res['success']     = true;
   $res['htmlContent'] = utf8_encode(nl2br($htmlContent));
   
   return $res;
}

function fileListDelete() {
	$res = array("success" => false);
	
	$fileIds = $_REQUEST['fileIds'];

	// try to delete file
	if (fileDelete($fileIds)) {
		$res["success"] = true;
	} else {
		setErrorMsg($res, "Konnte Datei nicht löschen.");
		return $res;
	}
	
	$res['test'] = $fileIds;
	return $res;
}

?>
