<?php
/**************************************************************************
 * $Id: matrix.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


require_once('lib/StudIPortFileTools.lib.php');
require_once('lib/StudIPortUploadTools.lib.php');
require_once('vendor/flexi/flexi.php');
require_once('lib/StudIPortDB.class.php');

// TODO TODO TODO
// do we have *any* input validation in here, i.e., can files be dropped by others
// or anything alike?

/** shows or hides rows/cols
 */
function matrixShowHidePart() {
	$res = array(
		"success" => false);

	// check if everything is provided
	if (!isset($_REQUEST["partId"]) || !isset($_REQUEST['isRow']) || !isset($_REQUEST['hide'])) {
		setErrorMsg($res, "Unvollständige Anfrage.");
		debuglog(_WARN, $GLOBALS['user']->id, "AJAX MATRIX: unvollstaendige Anfrage");
		return $res;
	}

	// get database connection
	$sql =& new DB_Seminar();

	// clean up
	$partId_esc 	= mysql_real_escape_string(intval($_REQUEST['partId']));
	$user_id_esc 	= mysql_real_escape_string($GLOBALS['user']->id);
	$isRow      	= intval($_REQUEST['isRow']);
	$hide			= intval($_REQUEST["hide"]);

	if ($isRow) {
		$mode = "comp";
	} else {
		$mode = "exp";
	}

	// hide
	if ($hide) {
		$sql->queryf("INSERT INTO studiport_matrix_%s_hide (%s_id, user_id)
		             VALUES ('%s','%s')", $mode, $mode, $partId_esc, $user_id_esc);

		$success = $sql->affected_rows();

		$sql->queryf("DELETE FROM studiport_file_assoc WHERE %s_id = '%s' AND file_id IN (SELECT file_id FROM studiport_file WHERE user_id='%s')",
					$mode, $partId_esc, $user_id_esc);

	// show
	} else {
		$sql->queryf("DELETE FROM studiport_matrix_%s_hide WHERE %s_id='%s' AND user_id='%s'", $mode, $mode, $partId_esc, $user_id_esc);

		$success = $sql->affected_rows();
	}

	// check result
	if (!$success) {
		setErrorMsg($res, "Datenbankoperation fehlgeschlagen.");
		debuglog(_WARN, $GLOBALS['user']->id, "AJAX MATRIX: Datenbankfehler");
		return $res;
	}

	$res['success'] = true;
	return $res;
}

/** removes a file from studiport
 */
function matrixRemoveFile() {
	$res = array(
		"success" => false,
		"lowPrioSuccess" => false,
		"usage" => null,
		"htmlContentList" => "",
		"htmlContentOverview" => "");

	// sanitize file key and check
	$fileKey = intval($_REQUEST['fileK']);
	if (empty($fileKey)) {
		setErrorMsg($res, "Keine Datei gewählt");
		return $res;
	}

	// obtain file association *BEFORE* deleting it
	$keys = fileGetAssoc($fileKey);
	if (empty($keys)) {
		setErrorMsg($res, "Kann Dateizuordnung nicht lesen");
		return $res;
	}

	// try to delete file
	if (fileDelete($fileKey)) {
		$res["success"] = true;
	} else {
		setErrorMsg($res, "Konnte Datei nicht löschen.");
		return $res;
	}

	// get updated matrix information
	if ($keys) {  // if file was not associated, don't reload matrix data
		$resExt_1 = helperGetCellList($keys["comp_id"], $keys["exp_id"], $keys["type_id"]);
		$resExt_2 = helperGetCellOverview($keys["comp_id"], $keys["exp_id"]);
		// prepare result
		if ($resExt_1["success"] && $resExt_2["success"]) {
			$res["lowPrioSuccess"] 	    = true;
			$res["htmlContentList"]     = utf8_encode($resExt_1["htmlContent"]);
			$res["num"]                 = utf8_encode($resExt_1["num"]);
			$res["htmlContentOverview"] = utf8_encode($resExt_2["htmlContent"]);
		} else {
			addWarning($res, $resExt_1["errorMsg"]);
			addWarning($res, $resExt_2["errorMsg"]);
		}
	}

	// update user quota (for file upload dialog)
	$res["usage"] = uploadGetUsageStat($GLOBALS['user']->id);

	return $res;
}

/** allocs a file to comp and exp
 */
function matrixAllocFile() {
	$res = array(
		"success" => false,
		"lowPrioSuccess" => false,
		"htmlContent" => "");

	if(fileAssoc($_REQUEST['fileK'], $_REQUEST['compK'], $_REQUEST['expK'])) {
		$res["success"] = true;
	} else {
		setErrorMsg($res, "Konnte Datei nicht zuordnen.");
		return $res;
	}

	$resExt = helperGetCellOverview($_REQUEST['compK'], $_REQUEST['expK']);
	$resExt_2 = filesGetNoAssocList();
	// prepare result
	if ($resExt["success"]) {
		$res["lowPrioSuccess"] 	= true;
		$res["htmlContent"] 	= utf8_encode($resExt["htmlContent"]);
		$res["htmlContentNoAssoc"]  = utf8_encode($resExt_2["htmlContent"]);
		$res["numNoAssoc"]          = $resExt_2["number"];
	}


	return $res;
}

/** releases a file from its allocation
 */
function matrixReleaseFile() {
	$res = array(
		"success" => false,
		"lowPrioSuccess" => false,
		"htmlContentList" => "",
		"htmlContentOverview" => "",
		"htmlContentNoAssoc" => "");

	// get current assoc, before it is deleted
	$keys = fileGetAssoc($_REQUEST['fileK']);

	// and then delete the assoc
	if (fileRelease($_REQUEST['fileK'])) {
		$res["success"] = true;
	} else {
		setErrorMsg($res, "Konnte nicht lösen.");
		return $res;
	}

	$resExt_1 = helperGetCellList($keys["comp_id"], $keys["exp_id"], $keys["type_id"]);
	$resExt_2 = helperGetCellOverview($keys["comp_id"], $keys["exp_id"]);
	$resExt_3 = filesGetNoAssocList();

	// prepare result
	if($resExt_1["success"] && $resExt_2["success"]) {
		$res["lowPrioSuccess"]      = true;
		$res["htmlContentList"]     = utf8_encode($resExt_1["htmlContent"]);
		$res["num"]                 = utf8_encode($resExt_1["num"]);
		$res["htmlContentOverview"] = utf8_encode($resExt_2["htmlContent"]);
		$res["htmlContentNoAssoc"]  = utf8_encode($resExt_3["htmlContent"]);
		$res["numNoAssoc"]          = $resExt_3["number"];
	} else {
		addWarning($res, $resExt_1["errorMsg"]);
		addWarning($res, $resExt_2["errorMsg"]);
		addWarning($res, $resExt_3["errorMsg"]);
	}

	return $res;
}

/** gets the entries for the filelist-box
 */
function matrixGetCell() {
	$res = array(
		"success" => false,
		"htmlContent" => "");

	$resExt = helperGetCellList($_REQUEST['compK'], $_REQUEST['expK'], $_REQUEST['typeK']);

	// prepare result
	if($resExt["success"]) {
		$res["success"]     = true;
		$res["htmlContent"] = utf8_encode($resExt["htmlContent"]);
	}

	return $res;
}

/** saves a comment
 */
function matrixCommentSave() {
	// prepare result
	$res = array(
		"success" => false,
		"text_int" => "",
		"text_ext" => "",
		"picture" => "no"
	);

	// check if parameters are valid
	// TODO we should use an external function here 
	// to check if expKey and compKey are valid
	if(!(is_numeric($_REQUEST["compKey"]) && is_numeric($_REQUEST["expKey"]))) {
		setErrorMsg($res, "Ungueltige Anfrage.");
		return $res;
	}

	// get database connection
	$sql =& new StudIPortDB();

	// if the comment is empty, simply remove the corresponding entry from the table!
	// otherwise, we'll trash up the table!
	if (strlen($_REQUEST["text_int"]) == 0 && strlen($_REQUEST["text_ext"]) == 0) {
		$sql->queryf("DELETE FROM studiport_matrix_cell_descr
			WHERE comp_id = %u
			AND exp_id = %u
			AND user_id = '%s'", $_REQUEST["compKey"], $_REQUEST["expKey"], $GLOBALS['user']->id);
		
		$res["success"] = true;
		
		return $res;
	}
		
	// check if entry exists
	$sql->queryf("SELECT comp_id
		FROM studiport_matrix_cell_descr
		WHERE comp_id = %u
		AND exp_id = %u
		AND user_id = '%s'
		LIMIT 0,1", $_REQUEST["compKey"], $_REQUEST["expKey"], $GLOBALS['user']->id);
	
	// prepare input
	$text_int = sanitizeInput($_REQUEST["text_int"]);
	$text_ext = sanitizeInput($_REQUEST["text_ext"]);
	$set = new MySQLSetClause();
	$set->set('text_int', $text_int);
	$set->set('text_ext', $text_ext);
	
	// if there is no entry, add one
	if ($sql->num_rows() == 0) {
		$set->set('user_id', $GLOBALS['user']->id);
		$set->set('comp_id', $_REQUEST["compKey"], false);
		$set->set('exp_id', $_REQUEST["expKey"], false);
		$set->set('created', "now()", false);
		
		$sql->query("INSERT INTO studiport_matrix_cell_descr SET ". $set->get());
		
		if($sql->affected_rows() == 1) {
			$res["success"] = true;
		}
	// update existing entry
	} else {
		$sql->queryf("UPDATE studiport_matrix_cell_descr SET ". $set->get(true) ."
			WHERE comp_id = %u
			AND exp_id = %u
			AND user_id = '%s'", $_REQUEST["compKey"],
			$_REQUEST["expKey"], $GLOBALS['user']->id);
		
		$res["success"] = true;
	}

	$res2 = matrixGetCellComment();
	$res["text_int"] = $res2["text_int"];
	$res["text_ext"] = $res2["text_ext"];
	$res["picture"] = $res2["picture"];

	return $res;
}

function matrixGetCellComment() {
	// prepare result
	$res = array(
		"success" => false,
		"text_int" => "",
		"text_ext" => "",
		"picture" => "no"
	);

	// check if parameters are valid
	// TODO we should use an external function here 
	// to check if expKey and compKey are valid
	if(!(is_numeric($_REQUEST["compKey"]) && is_numeric($_REQUEST["expKey"]))) {
		setErrorMsg($res, "Ungueltige Anfrage.");
		
		return $res;
	}

	// get database connection
	$sql =& new StudIPortDB();

	// get text
	$sql->queryf("SELECT text_int, text_ext
				FROM studiport_matrix_cell_descr
				WHERE comp_id = %u
				AND exp_id = %u
				AND user_id = '%s'
				LIMIT 0,1", $_REQUEST["compKey"], $_REQUEST["expKey"], $GLOBALS['user']->id);

	// if there is no entry return
	if ($sql->num_rows() == 0) {
		$res["success"] = true;
		
		return $res;
	}
	
	$sql->next_record();

	// calculate which picture to display
	$pictureIndex = 0;
	
	// add 1 if internal text is set
	if (strlen($sql->Record["text_int"]) > 0) {
		$pictureIndex += 1;
	}
	
	// add 2 if external text is set
	if (strlen($sql->Record["text_ext"]) > 0) {
		$pictureIndex += 2;
	}
	
	// kind of an enum
	$pictureNameEnum = array("no", "int", "ext", "yes");
	
	// prepare result data
	$res["picture"] = $pictureNameEnum[$pictureIndex];
	$res["text_int"] = utf8_encode($sql->Record["text_int"]);
	$res["text_ext"] = utf8_encode($sql->Record["text_ext"]);
	
	$res["success"] = true;

	return $res;
}


/** HELPER: returns html of a filelist-box
 * @param comp
 * @param exp
 * @param type
 */
function helperGetCellList($comp, $exp, $type) {
	// prepare output
	$res = array(
		"success" => false,
		"num" => 0,
		"htmlContent" => "");

	if(!(is_numeric($comp) && is_numeric($exp) && is_numeric($type))) {
		setErrorMsg($res, "Ungueltige Anfrage.");
		return $res;
	}

	// get database connection
	$sql =& new DB_Seminar();

	// clean up
	$comp_esc = mysql_real_escape_string(intval($comp));
	$exp_esc  = mysql_real_escape_string(intval($exp));
	$type_esc = mysql_real_escape_string(intval($type));

	$user_id_esc = mysql_real_escape_string($GLOBALS['user']->id);

	// get filetype name
	$sql->query("SELECT name
				FROM studiport_file_types
				WHERE type_id = ". $type_esc);

	$sql->next_record();
	$htmlContent = "<div class=\"boxHeader\">". $sql->Record['name'] ."</div><ul id='files_a'>";

	// get files
	$sql->query(
		'SELECT '.
		'  file_id, '.
		'  title, '.
		'  SUBSTRING(descr_ext, 1, '. MATRIX_FILE_DESCR_ABBR_LENGTH .') as descr_ext '.
		'FROM studiport_file file '.
		'  LEFT JOIN studiport_file_types types USING (type_id) '.
		'  LEFT JOIN studiport_file_assoc assoc USING (file_id) '.
		'WHERE file.user_id = \''. $user_id_esc .'\''.
		'  AND comp_id = '. $comp_esc .
		'  AND exp_id = '. $exp_esc .
		'  AND type_id = '. $type_esc);

	$url =& $GLOBALS['STUDIPORT_IMG_URI'];

	while ($sql->next_record()) {
		$link = getPluginLink('artefacts', array('file_id' => $sql->Record['file_id']));
		$sql->Record['descr_ext'] = textConvertNewline(textAbbreviate($sql->Record['descr_ext'], MATRIX_FILE_DESCR_ABBR_LENGTH));

		$htmlContent .= "<li><div class=\"listTitle\"><a href=\"". $link ."\">". $sql->Record['title'] ."</a></div><div class=\"listActions\">
						<a href=\"javascript:StudIPort.file.release(". $comp_esc .", ". $exp_esc .", ". $sql->Record['file_id'] .", ". $type_esc.")\"><img src=\"". $url ."/unlink.png\" title='Zuordnung aufheben' alt=\"Zuordnung aufheben\" width='16' height='16'></a>
						<a href=\"". $link ."\"><img src=\"". $url ."/edit.png\" title=\"bearbeiten\" alt='Datei bearbeiten' /></a>
						<a href=\"javascript:StudIPort.file.remove(". $comp_esc .", ". $exp_esc .", ". $sql->Record['file_id'] .", ". $type_esc.")\"><img src=\"". $url ."/trash.png\" alt=\"Datei löschen\" title='Datei löschen'></a></div><div class=\"listDescr\">". $sql->Record['descr_ext'] ."</div>
						</li>";
	}

	$htmlContent .= "</ul>";

	// prepare result
	$res["success"]     = true;
	$res["num"]         = $sql->num_rows();
	$res["htmlContent"] = $htmlContent;

	return $res;
}

/** HELPER: returns html of a tablecell
 * @param comp
 * @param exp
 */
function helperGetCellOverview($comp, $exp) {
	$res = array(
		"success" => false,
		"num" => 0,
		"htmlContent" => "");

	if(!(is_numeric($comp) && is_numeric($exp))) {
		setErrorMsg($res, "Ungueltige Anfrage.");
		return $res;
	}

	// get database connection
	$sql =& new DB_Seminar();

	// clean up
	$comp_esc = mysql_real_escape_string(intval($comp));
	$exp_esc = mysql_real_escape_string(intval($exp));

	$user_id_esc = mysql_real_escape_string($GLOBALS['user']->id);

	// get files
	$sql->query("SELECT type_id, types.name, types.picture, types.descr, COUNT(*) as num
				FROM studiport_file file
				LEFT JOIN studiport_file_types types USING (type_id)
				LEFT JOIN studiport_file_assoc assoc USING (file_id)
				WHERE file.user_id = '". $user_id_esc ."'
				AND comp_id = ". $comp_esc ."
				AND exp_id = ". $exp_esc ."
				GROUP BY type_id");

	$url =& $GLOBALS['STUDIPORT_IMG_URI'];

	while ($sql->next_record()) {
		$htmlContent .= "<img class='filetype' src=\"". $url ."/". $sql->Record['picture'] ."\" width=\"16\" height=\"16\" alt=\"". $sql->Record['descr'] ."\"
						onmouseover=\"StudIPort.matrix.openList('div_hoverBox', this, ". $comp_esc .", ". $exp_esc .", ". $sql->Record['type_id'] .")\"
						onmouseout=\"StudIPort.box.timedClose()\" /><img src=\"". $url ."/context-indicator.png\" class=\"contextIndicator\" /><span class='sub'>". $sql->Record['num'] ."</span>\n";
	}

	// prepare result
	$res["success"]     = true;
	$res["num"]         = $sql->num_rows();
	$res["htmlContent"] = $htmlContent;

	return $res;
}

?>
