<?php
/**************************************************************************
 * $Id: question.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


/* libs */
require_once('lib/StudIPortQuestionTools.lib.php');
require_once('lib/StudIPortUserTools.lib.php');


function questionDelete() {
	$res = array('success' => false);

	// question sanity check
	if (! validateQuestionAccess($_REQUEST['question_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// establish DB connection and sanitize input
	$db =& new DB_Seminar();
	$question_id_esc = mysql_real_escape_string(intval($_REQUEST['question_id']));

	// get view id before deleting anything
	$db->query("SELECT view_id FROM studiport_feedback_question WHERE question_id='$question_id_esc'");
	$db->next_record();
	$view_id = $db->f('view_id');

	// to follow foreign key constraints, delete from connected tables first
	$db->query("DELETE FROM studiport_feedback_response WHERE question_id='$question_id_esc'");
	$db->query("DELETE FROM studiport_feedback_request  WHERE question_id='$question_id_esc'");
	
	// TODO check whether SQL succeeded

	// all is well, so delete question
	$db->query("DELETE FROM studiport_feedback_question WHERE question_id='$question_id_esc'");
	if ($db->affected_rows() == 0) {
		setErrorMsg($res, "Feedback-Frage konnte nicht gelöscht werden");
		debuglog(_WARN, $GLOBALS['user']->id, "AJAX QUESTION: Feedback-Frage konnte nicht gelöscht werden");
		return $res;
	}

	// get updated list and return
	$res['htmlContent'] = utf8_encode(questionGetList($view_id, $GLOBALS['user']->id));
	$res['success'] = true;
	return $res;
}


/**
 * Creates an new question with default data and writes the
 * new data to
 */
function questionCreate() {
	$res = array('success' => false);
	$setClause = new MySQLSetClause();

	// view sanity check
        if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
                return $res;
        }	

	// validate name of question
	// TODO use generic checking here
	$text = sanitizeInput($_REQUEST['text']);
	$len  = strlen($text);
	if ($len < 10) {  // FIXME hard coding
		setErrorMsg($res, "Feedback-Frage '$text' zu kurz");
		return $res;
	} else if ($len > 1000) {  // FIXME hard coding
		setErrorMsg($res, "Feedback-Frage zu lang");
		return $res;
	}

	// get DB connection and sanitize input
	$db =& new DB_Seminar();

	// check, if view belongs to user!
	$view_id_esc = mysql_real_escape_string($_REQUEST['view_id']);
	$db->query("SELECT user_id FROM studiport_view WHERE view_id='". $view_id_esc ."'");
	if ($db->num_rows() != 1 || !$db->next_record() || $db->f('user_id') !== $GLOBALS['user']->id) {
		setErrorMsg($res, "Frage kann nicht zu fremder Ansicht hinzugefügt werden");
		return $res;
	}

	// default values for fields
	$setClause->set('text',        $text);
	$setClause->set('view_id',     $_REQUEST['view_id']);
	$setClause->set('expires',     "0000-00-00");
	$setClause->set('created',     "NOW()", false);
	$setClause->set('edited',      "NOW()", false);
	$setClause->set('public',  "N");

	// actually insert data into DB and verify success
	$db->query("INSERT INTO studiport_feedback_question SET ". $setClause->get());
	if ($db->affected_rows() == 0) {
		setErrorMsg($res, "Feedback-Frage konnte nicht aktualisiert werden");
		debuglog(_WARN, $GLOBALS['user']->id, "AJAX QUESTION: Feedback-Frage konnte nicht aktualisiert werden");
		return $res;
	}


	// create and return HTML content, so that the table/list can be updated
	$res['htmlContent'] = utf8_encode(questionGetList($_REQUEST['view_id'], $GLOBALS['user']->id));
	$res['success'] = true;
	return $res;
}


function questionSaveBasics() {
	$res = array('success' => false);
	$setClause = new MySQLSetClause();
	
	// question sanity check
	if (! validateQuestionAccess($_REQUEST['question_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// create DB connection and sanitize question id
	$db =& new DB_Seminar();
	$question_id = intval($_REQUEST['question_id']);
	$question_id_esc = mysql_real_escape_string($question_id);

	// TODO input validation
	// create update chunks and validate input
	$text =& $_REQUEST['text'];
	if (isset($text)) {
		$text = sanitizeInput($text);
		$setClause->set('text', $text);
	}
	
	// empty expiration => no expiration
	$expires =& $_REQUEST['expires'];
	if (isset($expires)) {
		$expires = sanitizeInput($expires);
		if ($expires === '---') {
			$setClause->set('expires', "0000-00-00");
		} else {
			list($day, $month, $year) = explode('.', $expires);
			if (checkdate($month, $day, $year)) {
				$setClause->set('expires', "$year-$month-$day");
			} else {
				// TODO error handling
			}
		}
	}

	// public question?
	$public = &$_REQUEST['public'];
	if (isset($public)) {
		if ($public == 'Y') {
			$setClause->set('public', 'Y');
		} else {
			$setClause->set('public', 'N');
		}
	}

	// TODO abort, if errors have been encountered
	//if (count($formErrors) > 0) {
	//	setErrorMsg($res, "$updateStr");
	//	return $res;
	//}
	
	// TODO verify this assumption!
	if ($setClause->count() == 0) {
		$res['success'] = true;  // nothing to change is a success
		return $res;
	}

	// update edited time
	$setClause->set('edited', "NOW()", false);
	
	// all is well, so save file details
	$db->query("UPDATE studiport_feedback_question SET ". $setClause->get() ." WHERE question_id='$question_id_esc'");
	// NOTE affected_rows is not helpful at this point, as it will return 0 in
	// case of an unchanged row

	// simply reload question data and utf-8 encode (for json)
	$question = questionGetBasics($question_id);
	if ($question != null) {
		array_utf8_encode($question);
	}

	// prepare result
	$res['formUpdates'] = $question;
	$res['success'] = true;
	return $res;
}


function questionReloadBasics() {
	$res = array('success' => false);

	// question sanity check
	if (! validateQuestionAccess($_REQUEST['question_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// get question basics
	$question = questionGetBasics(intval($_REQUEST['question_id']));
	if ($question != null) {
		array_utf8_encode($question);
	} else {
		setErrorMsg($res, "Feedback-Frage existiert nicht");
	}
	
	// prepare result
	$res['success'] = true;
	$res['formUpdates'] = $question;
	return $res;
}


function questionSaveRequest() {
	$res = array('success' => false);

	// question sanity check
	if (! validateQuestionAccess($_REQUEST['question_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// create DB connection and sanitize question id
	$db =& new DB_Seminar();
	$question_id_esc = mysql_real_escape_string(intval($_REQUEST['question_id']));

	// check array of provided user ids
	$newUsers = empty($_REQUEST['userlist']) ? array() : array_filter(array_unique(split(':', $_REQUEST['userlist'])));
	foreach ($newUsers as &$user) {
		if (!preg_match("/^[0-9a-fA-F]{32}$/", $user)) {
			// user-id is not StudIP conformant
			setErrorMsg($res, "Ungültige User-ID '$user' in Nutzerliste");
			return $res;
		}
	}

	// do not insert owner's user id to question
	$newUsers = array_diff($newUsers, array($GLOBALS['user']->id));

	// get current user id's from question and write them to an array
	$db->query("SELECT user_id FROM studiport_feedback_request WHERE question_id='$question_id_esc'");
	$oldUsers = array();
	while ($db->next_record()) {
		$oldUsers[] = $db->Record['user_id'];
	}

	// obtain users that are present in the old and in the new request list
	$keepUsers = array_intersect($oldUsers, $newUsers);

	// then identify the sets of users that must be added and removed
	$delUsers = array_diff($oldUsers, $keepUsers);
	$addUsers = array_diff($newUsers, $keepUsers);

	// delete obsolete users from request table
	if (count($delUsers) > 0) {
		$delLst = "'" . join("', '", $delUsers) . "'";
		$db->query("DELETE FROM studiport_feedback_request WHERE question_id='$question_id_esc' AND user_id IN (". $delLst .")");
		if ($db->affected_rows() != count($delUsers)) {
			addWarning($res, "Einige Feedback-Anfragen konnten nicht gelöscht werden");
		}
	}

	// insert new ones
	if (count($addUsers) > 0) {
		$addLst = "('". $question_id_esc ."', '". join("'), ('". $question_id_esc. "', '", $addUsers) ."')";
		$db->query("INSERT INTO studiport_feedback_request (question_id, user_id) VALUES ". $addLst);
		if ($db->affected_rows() != count($addUsers)) {
			addWarning($res, "Es konnten nicht alle Feedback-Anfragen gestellt werden");
		}
	}

	// if we have changed anything, update question edited timestamp
	if (count($delUsers) > 0 || count($addUsers) > 0) {
		$db->query("UPDATE studiport_feedback_question SET edited=NOW() WHERE question_id='". $question_id_esc ."'");
	}

	// prepare result
	$res['reqList'] = questionGetRequestList($_REQUEST['question_id']);
	$res['success'] = true;

	return $res;
}


function questionReloadRequest() {
	$res = array(success => false);

	// question sanity check
	if (! validateQuestionAccess($_REQUEST['question_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// prepare result
	$res['reqList'] = questionGetRequestList($_REQUEST['question_id']);
	$res['success'] = true;

	return $res;
}

?>
