<?php
/**************************************************************************
 * $Id: upload.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


/* load libs */
require_once('lib/StudIPortUploadTools.lib.php');

/**
 * get upload progress of given file
 */
function uploadGetProgress() {
	$res = array('success' => true);

	// check progress key
	$pKey = $_REQUEST['pKey'];

	if (empty($pKey)) {
		setErrorMsg($res, "Kein Progress-Schlüssel angegeben");
		return $res;
	}

	// check, whether apc is available at all
	if (function_exists("apc_fetch")) {
		$info = apc_fetch('upload_' . $pKey);
		// APC specific init
		if ($info) {
			$filename  = $info['filename'];
			$done      = $info['done'] ? true : false;
			$cancel    = $info['cancel'] ? true : false;
			$perc      = 0;
			$speed     = $info['rate'];
			$estTime   = -1;
			$startTime = $info['start_time'];
			$curSize   = $info['current'];
			$totalSize = $info['total'];
		}
	} else if (function_exists("uploadprogress_get_info")) {
		$info = uploadprogress_get_info($pKey);
		// UP specific init
		if ($info) {
			$filename  = $info['filename'];
			$done      = $info['files_uploaded'] ? true : false;
			$cancel    = false;
			$perc      = 0;
			$speed     = $info['speed_average'] / 1000;  // (opt.: speed_last)
			$estTime   = $info['est_sec'];
			$startTime = $info['time_start'];  // time_last usable somewhere?
			$curSize   = $info['bytes_uploaded'];
			$totalSize = $info['bytes_total'];
		}
	} else {
		$info = false;
		$res['noapc'] = true;
	}

	// get file info
	if ($info) {
/*
		$filename  = $info['filename'];
		$done      = $info['done'] ? true : false;
		$cancel    = $info['cancel'] ? true : false;
		$perc      = 0;
		$speed     = $info['rate'];
		$estTime   = -1;
		$curSize   = $info['current'];
		$totalSize = $info['total'];
*/

		// calculate percentage
		if ($totalSize > 0) {
			$perc = $curSize / $totalSize * 100;
		}

		// calculate current speed
		$dt = time() - $startTime;
		if (empty($speed) && $dt > 0 && !$done) {
			$speed = $curSize / 1024 / $dt;
		}

		// estimate time left
		if ($done || $cancel) {
			$estTime = 0;
		} else if ($speed > 0) {
			$estTime = ($totalSize - $curSize) / $speed;
		}

		// prepare results
		$res['progress'] = array(
			"pKey"      => $pKey,
			"filename"  => "$filename",
			"done"      => $done,
			"cancel"    => $cancel,
			"perc"      => intval($perc),
			"kps"       => round($speed, 1),
			"timeLeft"  => $estTime,
			"curSize"   => uploadSizeToString(intval($curSize), 1),
			"totalSize" => uploadSizeToString(intval($totalSize), 1)
		);

		// create a new pkey, if upload is done or was cancelled
		if ($done || $cancel) {
			$res['pKeyNext'] = createUploadKey();
		}
	} else {
		// TODO
	}

	return $res;
}

?>
