<?php
/**************************************************************************
 * $Id: view.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/



/* libs */
require_once('lib/StudIPortViewTools.lib.php');
require_once('lib/StudIPortUserTools.lib.php');
require_once('lib/StudIPortAccess.lib.php');


function viewSetViewedState() {
	$res = array('success' => false);

	// response sanity check (only receiver may delete)
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res, VIEW_ROLE_ALIEN)) {
		return $res;
	}

	// establish DB connection and sanitize input
	$db =& new DB_Seminar();
	$view_id_esc = mysql_real_escape_string(intval($_REQUEST['view_id']));
	$user_id_esc = mysql_real_escape_string($GLOBALS['user']->id);

	// prepare new state
	$state = ($_REQUEST['viewed']) ? 'Y' : 'N';

	// set viewed-state to true ('Y')
	$db->query("UPDATE studiport_view_access SET viewed='".$state."' WHERE user_id='". $user_id_esc ."' AND view_id='".$view_id_esc."'");
	// NOTE don't check result, as an unaltered state will return zero changed rows

	// we're done, so signal success and return
	$res['success'] = true;
	return $res;
}


function viewGetInfo() {
	$res = array('success' => false);

	$view_id = $_REQUEST['view_id'];

	// view sanity check
	if (!validateViewAccess($view_id, $GLOBALS['user']->id, $res, VIEW_ROLE_ALIEN)) {
		return $res;
	}

	$db =& new DB_Seminar();
	$view_id_esc = intval($view_id);
	$db->query("SELECT descr FROM studiport_view WHERE view_id='$view_id_esc'");

	$htmlContent = '<div class="boxHeader">Beschreibung</div>';

	$db->next_record();
	if (strlen($db->Record['descr']) > 0) {
		$htmlContent .= '<div class="boxContent">' . $db->Record['descr'] . '</div>';
	} else {
		$htmlContent .= '<div class="boxContent">Keine Beschreibung vorhanden.</div>';
	}

	$res['success']     = true;
	$res['htmlContent'] = utf8_encode(nl2br($htmlContent));

	return $res;
}


function viewDelete() {
	$res = array('success' => false);

	// view sanity check
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// establish DB connection and sanitize input
	$db =& new DB_Seminar();
	$view_id_esc = mysql_real_escape_string(intval($_REQUEST['view_id']));

	// to follow foreign key constraints, delete from connected tables first
	// delete
	// - responses to questions
	$db->query("DELETE FROM studiport_feedback_response WHERE question_id IN (SELECT question_id FROM studiport_feedback_question WHERE view_id='$view_id_esc')");
	// - requests for responses
	$db->query("DELETE FROM studiport_feedback_request WHERE question_id IN (SELECT question_id FROM studiport_feedback_question WHERE view_id='$view_id_esc')");
	// - questions of view
	$db->query("DELETE FROM studiport_feedback_question WHERE view_id='$view_id_esc'");
	// - view access
	$db->query("DELETE FROM studiport_view_access WHERE view_id='$view_id_esc'");
	// - view files associations
	$db->query("DELETE FROM studiport_view_files WHERE view_id='$view_id_esc'");
	
	// TODO check whether SQL succeeded

	// all is well, so delete view
	$db->query("DELETE FROM studiport_view WHERE view_id='$view_id_esc'");
	if ($db->affected_rows() == 0) {
		setErrorMsg($res, "Ansicht konnte nicht gelöscht werden");
		debuglog(_WARN, $GLOBALS['user']->id, "AJAX VIEW: Ansicht konnte nicht gelöscht werden");
		return $res;
	}

	$res['success'] = true;
	return $res;
}


/**
 * Creates an new view with default data and writes the
 * new data to
 */
function viewCreate() {
	$res = array('success' => false);
	$setClause = new MySQLSetClause();

	// validate name of view
	// TODO use generic checking here
	$name = sanitizeInput($_REQUEST['name']);
	$len  = strlen($name);
	if ($len < 3) {  // FIXME hard coding
		setErrorMsg($res, "Name der Ansicht '$name' zu kurz");
		return $res;
	} else if ($len > 50) {  // FIXME hard coding
		setErrorMsg($res, "Name der Ansicht zu lang");
		return $res;
	}

	// get DB connection and sanitize input
	$db =& new DB_Seminar();

	// default values for fields
	$setClause->set('name',        $name);
	$setClause->set('descr',       "");
	$setClause->set('user_id',     $GLOBALS['user']->id);
	$setClause->set('expires',     "0000-00-00");
	$setClause->set('created',     "NOW()", false);
	$setClause->set('edited',      "NOW()", false);
	$setClause->set('public',  "N");
	$setClause->set('access_code', viewCreateNewAccessCode());

	// actually insert data into DB and verify success
	$db->query("INSERT INTO studiport_view SET ". $setClause->get());
	if ($db->affected_rows() == 0) {
		setErrorMsg($res, "Ansicht konnte nicht aktualisiert werden");
		debuglog(_WARN, $GLOBALS['user']->id, "AJAX VIEW: Ansicht konnte nicht aktualisiert werden");
		return $res;
	}

	// TODO create and return HTML content, so that the table/list can be updated
	// without reloading the whole page
	$res['success'] = true;
	return $res;
}


function viewSaveBasics() {
	$res = array('success' => false);
	$setClause = new MySQLSetClause();
	
	// view sanity check
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
		setErrorMsg($res, "Zugriff verweigert");
		return $res;
	}

	// create DB connection and sanitize view id
	$db =& new DB_Seminar();
	$view_id = intval($_REQUEST['view_id']);
	$view_id_esc = mysql_real_escape_string($view_id);

	// TODO input validation
	// create update chunks and validate input
	$name =& $_REQUEST['name'];
	if (isset($name)) {
		$name = sanitizeInput($name);
		$setClause->set('name', $name);
	}
	
	// TODO input validation
	$descr =& $_REQUEST['descr'];
	if (isset($descr)) {
		$descr = sanitizeInput($descr);
		$setClause->set('descr', $descr);
	}
	
	// empty expiration => no expiration
	$expires =& $_REQUEST['expires'];
	if (isset($expires)) {
		$expires = sanitizeInput($expires);
		if ($expires === '---') {
			$setClause->set('expires', "0000-00-00");
		} else {
			list($day, $month, $year) = explode('.', $expires);
			if (checkdate($month, $day, $year)) {
				$setClause->set('expires', "$year-$month-$day");
			} else {
				// TODO error handling
			}
		}
	}

	$public = &$_REQUEST['public'];
	if (isset($public)) {
		if ($public == 'Y') {
			$setClause->set('public', 'Y');
		} else {
			$setClause->set('public', 'N');
		}
	}

	$access_code_renew = &$_REQUEST['access_code_renew'];
	if (! empty($access_code_renew)) {
		$setClause->set('access_code', viewCreateNewAccessCode());
	}

	// TODO abort, if errors have been encountered
	//if (count($formErrors) > 0) {
	//	setErrorMsg($res, "$updateStr");
	//	return $res;
	//}
	
	// TODO verify this assumption!
	if ($setClause->count() == 0) {
		$res['success'] = true;  // nothing to change is a success
		return $res;
	}

	// update edited time
	$setClause->set('edited', "NOW()", false);
	
	// all is well, so save file details
	$db->query("UPDATE studiport_view SET ". $setClause->get() ." WHERE view_id='$view_id_esc'");
	// NOTE affected_rows is not helpful at this point, as it will return 0 in
	// case of an unchanged row

	// simply reload view data and utf-8 encode (for json)
	$view = viewGetBasics($view_id);
	if ($view != null) {
		$view['access_code_renew'] = 'N';
		array_utf8_encode($view);
	}

	// prepare result
	$res['formUpdates'] = $view;
	$res['success'] = true;
	return $res;
}


function viewReloadBasics() {
	$res = array('success' => false);

	// view sanity check
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// get view basics
	$view = viewGetBasics(intval($_REQUEST['view_id']));
	if ($view != null) {
		$view['access_code_renew'] = 'N';	
		array_utf8_encode($view);
	} else {
		setErrorMsg($res, "Ansicht existiert nicht");
	}
	
	// prepare result
	$res['success'] = true;
	$res['formUpdates'] = $view;
	return $res;
}


function viewSaveAccess() {
	$res = array('success' => false);

	// view sanity check
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// create DB connection and sanitize view id
	$db =& new DB_Seminar();
	$view_id_esc = mysql_real_escape_string(intval($_REQUEST['view_id']));

	// check array of provided user ids
	$newUsers = empty($_REQUEST['access']) ? array() : array_filter(array_unique(split(':', $_REQUEST['access'])));
	foreach ($newUsers as &$user) {
		if (!preg_match("/^[0-9a-fA-F]{32}$/", $user)) {
			// user-id is not StudIP conformant
			setErrorMsg($res, "Ungültige User-ID '$user' in Nutzerliste");
			return $res;
		}
	}

	// do not insert owner's user id to view
	$newUsers = array_diff($newUsers, array($GLOBALS['user']->id));

	// get current user id's from view and write them to an array
	$db->query("SELECT user_id FROM studiport_view_access WHERE view_id='$view_id_esc'");
	$oldUsers = array();
	while ($db->next_record()) {
		$oldUsers[] = $db->Record['user_id'];
	}

	// obtain users that are present in the old and in the new access list
	$keepUsers = array_intersect($oldUsers, $newUsers);

	// then identify the sets of users that must be added and removed
	$delUsers = array_diff($oldUsers, $keepUsers);
	$addUsers = array_diff($newUsers, $keepUsers);

	// delete obsolete users from access table
	if (count($delUsers) > 0) {
		$delLst = "'" . join("', '", $delUsers) . "'";

		// delete feedback requests associated with this user
		$db->query("DELETE FROM studiport_feedback_request ".
		           "WHERE user_id IN (". $delLst .") ".
		           "  AND question_id IN ( ".
		           "      SELECT question_id ".
		           "      FROM studiport_feedback_question ".
		           "      WHERE view_id='". $view_id_esc ."')");

		// delete access
		$db->query("DELETE FROM studiport_view_access WHERE view_id='$view_id_esc' AND user_id IN (". $delLst .")");
		if ($db->affected_rows() != count($delUsers)) {
			addWarning($res, "Einige Berechtigungen konnten nicht aufgehoben werden");
		}
	}

	// insert new ones
	if (count($addUsers) > 0) {
		$addLst = "('". $view_id_esc ."', '". join("'), ('". $view_id_esc. "', '", $addUsers) ."')";
		$db->query("INSERT INTO studiport_view_access (view_id, user_id) VALUES ". $addLst);
		if ($db->affected_rows() != count($addUsers)) {
			addWarning($res, "Es konnten nicht alle Berechtigungen erteilt werden");
		}
	}

	// if we have changed anything, update view edited timestamp
	if (count($delUsers) > 0 || count($addUsers) > 0) {
		$db->query("UPDATE studiport_view SET edited=NOW() WHERE view_id='". $view_id_esc ."'");
	}

	// get updated list of users with access
	$accessLst = getStudIPortViewAccessList($_REQUEST['view_id']);
	$htmlContent = "";
        foreach ($accessLst as $person) {
                $htmlContent .= getUserListEntry($person);
        }
	
	// prepare result
	$res['success']     = true;
	$res['htmlContent'] = $htmlContent;

	return $res;
}


function viewReloadAccess() {
	$res = array(
		success => false,
		htmlContent => '');

	// view sanity check
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
		return $res;
	}

	// get view access
	$accessLst = getStudIPortViewAccessList($_REQUEST['view_id']);

	// now create the list
	$htmlContent = "";
        foreach ($accessLst as $person) {
                $htmlContent .= getUserListEntry($person);
        }

	// prepare result
	$res['success']     = true;
	$res['htmlContent'] = $htmlContent;

	return $res;
}


function viewSaveFiles() {
	$res = array('success' => false);

	// view sanity check
	$view_id = intval($_REQUEST['view_id']);
	$user_id = $GLOBALS['user']->id;
	if (! validateViewAccess($view_id, $user_id, $res)) {
		return $res;
	}

        // create DB connection and sanitize view id
        $db =& new DB_Seminar();
        $view_id_esc = mysql_real_escape_string($view_id);
        $user_id_esc = mysql_real_escape_string($user_id);

        // convert selected files into an array
        $selFiles  = empty($_REQUEST['files']) ? array() : array_filter(array_unique(split(':', $_REQUEST['files'])));

	// get all files belonging to user
	$db->query("SELECT file_id FROM studiport_file WHERE user_id='". $user_id_esc ."'");
	$userFiles = array();
	while ($db->next_record()) {
		array_push($userFiles, $db->f("file_id"));
        }

	// now verify that all selected files really belong to the user
	foreach ($selFiles as $f) {
		if (! in_array($f, $userFiles)) {
			setErrorMsg($res, "Ungültige Datei");
			return $res;
		}
	}

        // get currently selected files
        $db->query("SELECT file_id FROM studiport_view_files WHERE view_id='$view_id_esc'");
        $oldFiles = array();
        while ($db->next_record()) {
                array_push($oldFiles, $db->f('file_id'));
        }

        // obtain users that are present in the old and new selection
        $keepFiles = array_intersect($oldFiles, $selFiles);

        // then identify the sets of files that must be added and removed
        $delFiles = array_diff($oldFiles, $keepFiles);
        $addFiles = array_diff($selFiles, $keepFiles);

        // delete obsolete users from access table
        if (count($delFiles) > 0) {
                $delLst = "'" . join("', '", $delFiles) . "'";
                $db->query("DELETE FROM studiport_view_files WHERE view_id='$view_id_esc' AND file_id IN (". $delLst .")");
                if ($db->affected_rows() != count($delFiles)) {
                        addWarning($res, "Einige Zuordnungen konnten nicht aufgehoben werden");
                }
        }

        // insert new ones
        if (count($addFiles) > 0) {
                $addLst = "('". $view_id_esc ."', '". join("'), ('". $view_id_esc. "', '", $addFiles) ."')";
                $db->query("INSERT INTO studiport_view_files (view_id, file_id) VALUES ". $addLst);
                if ($db->affected_rows() != count($addFiles)) {
                        addWarning($res, "Es konnten nicht alle neuen Zuordnungen durchgeführt werden");
                }
        }

        // if we have changed anything, update view edited timestamp
        if (count($delFiles) > 0 || count($addFiles) > 0) {
                $db->query("UPDATE studiport_view SET edited=NOW() WHERE view_id='". $view_id_esc ."'");
        }

        // return updated list of files
        return viewReloadFiles($res);
}



function viewReloadFiles() {
	$res = array(success => false);

	// view sanity check
	if (! validateViewAccess($_REQUEST['view_id'], $GLOBALS['user']->id, $res)) {
			return $res;
	}

	// get ID's of files included in this view
	$db =& new DB_Seminar();
	$view_id_esc = mysql_real_escape_string($_REQUEST['view_id']);
	$db->query("SELECT file_id FROM studiport_view_files WHERE view_id='". $view_id_esc ."'");

	// now turn result set into a list/an array
	$fileList = array();
	
	while ($db->next_record()) {
		array_push($fileList, $db->f('file_id'));
	}

	// prepare result
	$res['success']  = true;
	$res['fileList'] = $fileList;

	return $res;
}
?>
