/**************************************************************************
 * $Id: ajax.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.ajax =
{
	/** perform an ajax-request
	 *
	 * @param params to be passed to server
	 * @param success function to be called if request is successful
	 * @param failure function to be called if request fails
	 */
	request : function(params, success, failure)
	{
		//  add (box) session id to params
		params.session = StudIPort.box.sessionID;
		
		// start ajax request
		new Ajax.Request(StudIPort.urlAjaxServer, {
			method: "post",
			requestHeaders: {Accept: 'application/json'},
			parameters: params,
			onSuccess:  function(transport)
			{
				// get response
				res = transport.responseJSON;
				
				// decide which function to call
				if (res.success) {
					funcToCall = success;
				} else {
					funcToCall = failure;
				}
				
				// check if it's really a function
				if (typeof funcToCall == 'function')
				{
					// call it
					funcToCall(res, params);
				}
			},
			onFailure: function(transport)
			{
				errorMsg = "Es ist ein Fehler aufgetreten!";
				
				// catch special error codes
				if (400 <= transport.status)
				{
					errorMsg += "<br /><i>Beschreibung: Konnte keine Verbindung zum Server herstellen</i>";
				} else {
					errorMsg += "<br /><i>Beschreibung: unbekannter Fehler, versuchen Sie es spter erneut</i>";
				}

				// do something to inform user
				StudIPort.msgBox.update("superSpecialErrorBox", errorMsg, StudIPort.msgBox.ERROR);
				
				// call failure
				if (typeof failure == 'function') 
				{
					failure(null, params, false);
				}
			}
		});
	}
};
