/**************************************************************************
 * $Id: box.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.box =
{
	closeDelay  :  300,  // closing delay [ms]
	closeTimer  : null,  // reference to timer for delaying box closing
	openDelay   :  250,  // opening delay [ms]
	openTimer   : null,  // reference to timer for delaying box opening
	box         : null,  // reference to currently open box
	curObj      : null,  // currently displayed source
	keepOpen    : false, // don't close box
	vShift      : 25,    // vertical shift for boxes (to keep distance to text/object)
	sessionID   : 0,     // a session id to keep track of box openings
	sessionCnt  : 0,     //

	// FIXME this is only a workaround for the problem of delayed box opening
	// In fact, box opening should be provided with a callback function that is called as soon as the
	// box actually opens. This ensures that AJAX calls are only made, if the user really wants to open a
	// box, i.e., she settles on an icon!
	/**
	 * @param content html content of the box
	 * @param id of the box
	 */
	contentBuffer  : null,
	contentSession : null,

	insertContent : function(content, session) {
		if (this.sessionID == session) {
			this.contentBuffer  = content;
			if (this.contentSession == session) {
				this.insertContentDo();
			}
			this.contentSession = session;

			return true;
		} else {
			return false;
		}
	},

	/**
         * actually insert content into the box, if one exists
	 */
	insertContentDo : function()
	{
		if (this.contentBuffer && this.contentSession == this.sessionID) {
			this.box.update(this.contentBuffer);
			this.contentBuffer = null;

			// this is a workaroung for long document titles in firefox
			// other browsers have a better default behaviour or use css property word-wrap (eg. ie)
			// This should work for: Firefox (workaround), Chrome, Opera, IE
			if (checkBrowserName('Firefox')) {
				var walker = document.createTreeWalker(this.box, NodeFilter.SHOW_TEXT, null, false);

				while (walker.nextNode()) {
					var node = walker.currentNode;
					if (node.parentNode.parentNode.className == "listTitle") {
						node.nodeValue = node.nodeValue.split('').join(String.fromCharCode('8203'));
					}
				}
			}

			// get viewport bottom position and re-position box, if not enough space below
			if (window.innerHeight != 'undefined') {
				vpBottom = window.innerHeight + document.viewport.getScrollOffsets().top;
				if (this.box.cumulativeOffset().top + this.box.getHeight() > vpBottom) {
					this.box.style.top = (this.box.cumulativeOffset().top - this.box.getHeight() - 1.5 * this.vShift) + "px";
				}
			}
		} else {
			this.contentSession = this.sessionID;  // the content can now be updated; this must be done to ensure that insertContent automatically updates an open box
		}
	},

	open : function(id, obj, defcont)
	{
		// make sure the object exists
		obj = $(obj);
		if (obj == null) {
			return null;
		}

		// stop timer
		this.cancelClose();
		this.cancelOpen();

		// don't act, if same box
		if (obj == this.curObj) {
			return this.sessionId;
		}

		fct = "StudIPort.box.openNow('"+ id +"','"+ obj +"',"+ defcont +")";
//		this.openTimer = window.setTimeout(fct, this.openDelay);
		this.openTimer = window.setTimeout(function x() { StudIPort.box.openNow(id, obj, defcont)} , this.openDelay);
	  	
		// increase session
		this.sessionID = this.sessionCnt;
		return this.sessionCnt++;
	},

	/** show a box below given object
	 *
	 * @param id of the div to open
	 * @param obj below which the box should be displayed
	 * @param defcont insert default content (boolean)
	 */
	// NOTE *must not* be called directly from outside!
	openNow : function(id, obj, defcont)
	{
		// make sure the object exists
		obj = $(obj);
		if (obj == null) {
			return;
		}

		// stop timer
		this.cancelClose();

		// don't act, if same box
		if (this.box && obj == this.curObj) {
			return;
		}

		// close old div, if opened
		if (this.box) {
			this.box.hide();
		}
		
		offset = obj.cumulativeOffset();

		topOffset = offset[1];
		leftOffset = offset[0];

		// insert default content
		this.box = $(id);
		if (defcont) {
			this.defaultContent();
		}

		// get viewport right position
		// do some special stuff for MSIE
		if (checkBrowserName('MSIE')) {
			// IE is somewhat strange and works with the bottom corner, so
			// we must manually subtract the boxes height
			topOffset -= this.box.style.height;

			// IE6
			vpRight = document.documentElement.clientWidth;

			if (vpRight == 0) {
				// even older IE or quirks mode
				vpRight = document.getElementsByTagName('body')[0].clientWidth;
			}
		} else {
			// this works for mozilla,chrome, opera, IE7 or newer
			vpRight  = document.viewport.getWidth()  + document.viewport.getScrollOffsets().left;
		}

		// horizontal box position according to window width
		if (leftOffset + this.box.getWidth() > vpRight) {
			leftOffset -= this.box.getWidth() - obj.getWidth();

			if (leftOffset < 0) {
				leftOffset = 0;
			}
		}

		// vertical box position
		topOffset += -this.box.style.height + this.vShift;

		// position box
		this.box.style.left = leftOffset + "px";
	  	this.box.style.top  = topOffset + "px";
	  	
		// update content
		this.insertContentDo();
	  	
		// show the box
		//this.box.show();
		if (obj != this.curObj) {
			this.curObj = obj;
			Effect.Appear(this.box, { duration: 0.3 });
		};
	},

	/** cancels the timer that has been set to close the currently open box.
	 */
	cancelOpen : function()
	{
		if (this.openTimer) {
			window.clearTimeout(this.openTimer);
			this.openTimer = null;
		}
	},

	/** hides the box
	 */
	closeNow : function()
	{
		// cancel opening timer
		this.cancelOpen();

		// cancel pending close timer, this is done for safety in case this function is called directly
		this.cancelClose();

		if (this.box && !this.keepOpen) {
			//this.box.hide();
			//Effect.Shrink(this.box, { duration: 0.3 });
			Effect.Fade(this.box, { duration: 0.3 });
			this.box    = null;
			this.curObj = null;
		}
	},

	/** starts a timer to close the box
	 */
	close : function()
	{
		// be sure there's now other closing timer
		if (! this.keepOpen && this.closeTimer == null) {
			this.cancelOpen();
			// and then set the timeout for closing
			this.closeTimer = window.setTimeout("StudIPort.box.closeNow()", this.closeDelay);
		}
	},

	/** cancels the timer that has been set to close the currently open box.
	 */
	cancelClose : function()
	{
		if (this.closeTimer) {
			window.clearTimeout(this.closeTimer);
			this.closeTimer = null;
		}
	},

	/** resets the box to its default content
	 */
	defaultContent : function()
	{
		//this.box.update('<div class="boxHeader">L&auml;dt...</div>');
		//this.contentBuffer = '<div class="boxHeader">L&auml;dt...</div>';
	}
};
