/**************************************************************************
 * $Id: cache.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.cache =
{
	data : {},	// data storage
	
	/** builds an index from given keys
	 *
	 * @param key_1
	 * @param key_2
	 * @param key_3 optional
	 */
	buildIndex : function(key_1, key_2, key_3)
	{
		if (key_3 == null) {
			key_3 = "N";
		}
		
		return key_1 + "," + key_2 + "," + key_3;
	},
	
	/** checks wether a cached entry exists for given key
	 * @param indexStr cache index
	 */
	exists : function(indexStr)
	{
		return (this.data[indexStr] == undefined) ? false : true;
	},
	
	/** updates a cache entry or inserts a new one
	 * @param indexStr cache index
	 * @param content to store in cache
	 */
	update : function(indexStr, content)
	{
		this.data[indexStr] = content;
	},

	/** returns content of requested cache entry
	 * @param indexStr cache index
	 */
	get : function(indexStr)
	{
		return this.exists(indexStr) ? this.data[indexStr] : '';
	},
	
	/** deletes cache entry
	 * @param indexStr cache index
	 */
	del : function(indexStr)
	{
		if (this.exists(indexStr)) {
			delete this.data[indexStr];
		}
	}
};
