/**************************************************************************
 * $Id: msgBox.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.msgBox =
{
	WAITING : 0,  // idle or waiting for AJAX response
	CHANGED : 1,  // data has been changed
	OK      : 2,  // AJAX response is ok and the request was successful
	WARNING : 3,  // AJAX request succeeded, but warnings were issued
	ERROR   : 4,  // AJAX request could not be handled by server
	COLORS  : Array('#ced2d8', '#fff0b0', '#44a044', '#d08000', '#cc4444'),
	
	/**
	 * update content and display style of a message box
	 *
	 * @param msgBox  handle to or identifier of a message box
	 * @param msgs    message (or array of messages) to be displayed
	 * @param msgType type of message(s), e.g. MSGBOX_OK
	 */
	update : function(id, msgs, msgType)
	{
		// sanity checks
		if (msgType < this.WAITING || msgType > this.ERROR)
		{
			return;
		}

		// get box
		obj = $(id);
		
		if (obj == null)
		{
			return;
		}

		// set style
		//msgBox.className = style;

		// update msgbox content
		if (msgs instanceof Array)
		{	
			obj.update(msgs.join(' '));
		}
		else
		{
			if (msgs.length == 0)
			{
				// workaround, content isn't cleared else
				// this also ensures that the box doesn't shrink!
				msgs = "&nbsp;";
			}
			
			obj.update(msgs);
		}

		// highlight message box, if state is not waiting
		if (msgType == this.WAITING)
		{
			obj.style.backgroundColor = this.COLORS[this.WAITING];
		}
		else if (msgType == this.CHANGED)
		{
			obj.style.backgroundColor = this.COLORS[this.CHANGED];
		}
		else
		{
			new Effect.Highlight(obj, {
				startcolor   : this.COLORS[msgType],
				endcolor     : this.COLORS[this.WAITING],
				restorecolor : this.COLORS[this.WAITING],
				duration     : 2.5
			});
		}
	}
};