/**************************************************************************
 * $Id: question.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.question =
{
	//hideDelay : 250,
	//hideTimer : null,

	/**
	 * deletes a question using AJAX
	 *
	 * @param questId  identifier of the question (as in database)
	 */
	del : function(questId)
	{
		question = "Diese Frage wirklich löschen?";
		ask      = confirm(question);
		
		StudIPort.ajax.request({cmd: 'questionDelete', question_id: questId}, this.deleteSuccess, this.deleteFailure);
	},
	
	deleteSuccess : function(res, par)
	{
		dst = $('StudIPortQuestionList');
		if (dst != null) {
			dst.update(res.htmlContent);
		}
	},
	
	deleteFailure : function(res, par)
	{
		// TODO more appropriate way of displaying error message required!
		alert(res.errorMsg);
	},
	
	/**
	 * creates a new question using AJAX
	 */
	create : function(viewId, questionText)
	{
		StudIPort.ajax.request({cmd: 'questionCreate', view_id: viewId, text: questionText}, this.createSuccess, this.createFailure);
	},
	
	createSuccess : function(res, par)
	{
		dst = $('StudIPortQuestionList');
		if (dst != null) {
			dst.update(res.htmlContent);
		}
	},
	
	createFailure : function(res, par)
	{
		// TODO more appropriate way of displaying error message required!
		alert(res.errorMsg);
	},
	
	/**
	 * saves the feedback request form via an AJAX request
	 *
	 * @param questId identifier of the question (as in database)
	 */
	saveRequestForm : function(questId)
	{
		// serialize data
		ch = $('StudIPortRequestListForm').getInputs('checkbox');
                userIds = '';
		if (ch.length > 0) {
                	userIds = ch.map(function(item) {
				return $F(item);
			}).join(':');
		}                
		StudIPort.msgBox.update('questionRequestFormMsgBox', "&Uuml;bermittle Daten ...", StudIPort.msgBox.WAITING);
	
		// send data via ajax request
		StudIPort.ajax.request({cmd: 'questionSaveRequest', question_id: questId, userlist: userIds}, this.saveRequestFormSuccess, this.saveRequestFormFailure);
	},
	
	saveRequestFormSuccess : function(res, par)
	{
		if (res.warnings != null) {
			StudIPort.msgBox.update('questionRequestFormMsgBox', res.warnings, StudIPort.msgBox.WARNING, true);
		} else {
			StudIPort.msgBox.update('questionRequestFormMsgBox', 'Daten gespeichert', StudIPort.msgBox.OK, true);
		}
		
		// reset list content and recreate sortable
		StudIPort.updateCheckboxes('StudIPortRequestListForm', res.reqList);
	},
	
	saveRequestFormFailure : function(res, par)
	{
		StudIPort.msgBox.update('questionRequestFormMsgBox', res.errorMsg, StudIPort.msgBox.ERROR, true);
	},
	
	/**
	 * reloads the current request list of the provided question via an AJAX request
	 *
	 * @param questId identifier of the question (as in database)
	 */
	reloadRequestForm : function(questId) {
		StudIPort.msgBox.update('questionRequestFormMsgBox', 'Hole Daten ...', StudIPort.msgBox.WAITING);
		
		StudIPort.ajax.request({cmd: 'questionReloadRequest', question_id: questId}, this.reloadRequestFormSuccess, this.reloadRequestFormFailure);
	},
	
	reloadRequestFormSuccess : function(res, par)
	{
		// reset list content and recreate sortable
		StudIPort.updateCheckboxes('StudIPortRequestListForm', res.reqList);
		
		// update message box
		if (res.warnings == null) {
			StudIPort.msgBox.update('questionRequestFormMsgBox', 'Daten zur&uuml;ckgesetzt', StudIPort.msgBox.OK, true);
		} else {
			StudIPort.msgBox.update('questionRequestFormMsgBox', res.warnings, StudIPort.msgBox.WARNING, true);
		}
	},
	
	reloadRequestFormFailure : function(res, par)
	{
		StudIPort.msgBox.update('questionRequestFormMsgBox', res.errorMsg, StudIPort.msgBox.ERROR, true);
	},

	show : function(questId) {
		txt  = $('quest_text_' + questId);
                abbr = $('quest_text_abbr_'  + questId);
                btn  = $('quest_btn_'  + questId);
		lst  = $('re_list_' + questId);

                if (txt.getStyle("display") == "none") {
                        txt.show();
                        abbr.hide();
			lst.show();
                        btn.src = btn.src.sub('expand', 'collapse');
		} else {
                        txt.hide();
                        abbr.show();
			lst.hide();
                        btn.src = btn.src.sub('collapse', 'expand');
		}
	}
};
