/**************************************************************************
 * $Id: response.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.response =
{
	//hideDelay : 250,
	//hideTimer : null,

	/**
	 * deletes a question using AJAX
	 *
	 * @param questId  identifier of the question (as in database)
	 */
	del : function(reId)
	{
		question = "Diese Frage wirklich löschen?";
		ask      = confirm(question);
		
		StudIPort.ajax.request({cmd: 'responseDelete', response_id: reId}, this.deleteSuccess, this.deleteFailure);
	},
	
	deleteSuccess : function(res, par)
	{
		dst = $('StudIPortResponseList');
		if (dst != null) {
			dst.replace(res.htmlContent);
		}
	},
	
	deleteFailure : function(res, par)
	{
		// TODO more appropriate way of displaying error message required!
		alert(res.errorMsg);
	},
	
	/**
	 * creates a new question using AJAX
	 */
	showCreate : function(questId) {
		box = $('box_re_new_' + questId);
		btn = $('btn_re_new_' + questId);
		if (box) {
			box.show();
		}
		if (btn) {
			btn.hide();
		}
		// complete question
		StudIPort.question.show(questId);
	},

	discardCreate : function(questId) {
		box  = $('box_re_new_'  + questId);
		btn  = $('btn_re_new_'  + questId);
		name = $('name_re_new_' + questId);
		txt  = $('re_new_text_' + questId);
		
		if (name) {
			name.clear(); // does not work with Chromium 4.0.294.0 (35786)
		}
		if (txt) {
			txt.clear();
		}
		if (box) {
			box.hide();
		}
		if (btn) {
			btn.show();
		}
		// complete question
		StudIPort.question.show(questId);
	},


	create : function(questId)
	{
		responseName = $('name_re_new_' + questId);
		responseText = $F('re_new_text_' + questId);
		
		author = responseName === null ? '' : responseName.getValue();
		
		StudIPort.ajax.request({cmd: 'responseCreate', question_id: questId, text: responseText, author: author}, this.createSuccess, this.createFailure);
	},
	
	createSuccess : function(res, par)
	{
		questId = par.question_id;
		// delete input field
		box = $('box_re_new_' + questId);
		btn = $('btn_re_new_' + questId);
		if (box) {
			box.remove();
		}
		if (btn) {
			btn.remove();
		}

		// add new response to response list (at the beginning)
		lst = $('re_list_' + questId);
		if (lst) {
			lst.insert({top: res.htmlContent});
			lst.show();
		}

		// there is a response now, so toggle state
		state = $('quest_state_tb_' + questId);
		if (state) {
			state.src = state.src.sub('none', 'old');
		}
		
		// there is a response now, so toggle state
		state = $('quest_state_' + questId);
		if (state) {
			state.src = state.src.sub('yes', 'no');
		}
	},
	
	createFailure : function(res, par)
	{
		// TODO more appropriate way of displaying error message required!
		alert(res.errorMsg);
	},

	/* view (open) a response and flip the viewed bit in the DB */
	view : function(reId)
	{
                txt  = $('re_text_' + reId);
                abbr = $('re_text_abbr_'  + reId);
                //btn = $('re_btn_'  + reId);

                if (txt.getStyle("display") == "none") {
                        txt.show();
                        abbr.hide();
                        //btn.src = btn.src.sub('expand', 'collapse');
			// flip viewed-state in DB
			// TODO only do this, if the state isn't "new" = (unviewed)
			StudIPort.ajax.request({cmd: 'responseSetViewedState', response_id: reId, viewed: 1}, this.viewSuccess, this.viewFailure);
                } else {
                        txt.hide();
                        abbr.show();
                        //btn.src = btn.src.sub('collapse', 'expand');
                }

	},

	viewSuccess : function(res, par)
	{
		state = $('re_state_' + par.response_id);
		state.src = state.src.sub('yes', 'no');
	},
	
	viewFailure : function(res, par)
	{
		/* silently ignore this at the moment */
	},


	/* toggle state of message (viewed / not viewed) */
	stateToggle : function(reId)
	{
                state = $('re_state_' + reId);
		if (state == null) {
			return;
		}

		viewedState = (state.src.indexOf("yes") == -1) ? 0 : 1;
		StudIPort.ajax.request({cmd: 'responseSetViewedState', response_id: reId, viewed: viewedState}, this.stateToggleSuccess, this.stateToggleFailure);
	},

	stateToggleSuccess : function(res, par)
	{
		state = $('re_state_' + par.response_id);
		if (state == null) {
			return;
		}

		// indicate new state
		if (par.viewed != 0) {
			state.src = state.src.sub('yes', 'no');
		} else {
			state.src = state.src.sub('no', 'yes');
		}
	},
	
	stateToggleFailure : function(res, par)
	{
		/* silently ignore this at the moment */
	},

////////
////////
// new stuff
	show : function(response_id) {
		txt  = $('re_text_' + response_id);
		abbr = $('re_text_abbr_' + response_id);
		if (txt) {
			txt.show();
		}
		if (abbr) {
			abbr.hide();
		}
	},

	hide : function(response_id) {
		txt  = $('re_text_' + response_id);
		abbr = $('re_text_abbr_' + response_id);
		if (txt) {
			txt.hide();
		}
		if (abbr) {
			abbr.show();
		}
	}
};
