/**************************************************************************
 * $Id: upload.js 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


StudIPort.Upload = function(prefix)
{
	// initialization of variables
	this.prefix       = prefix;
	this.isUploading  = false;
	this.request      = {};
	this.pollInterval = 0.5;  // polling of updated upload progress [s]
	this.poller       = null; // the polling object
}
	
	
/**
 * start a file upload
 */
StudIPort.Upload.prototype.init = function()
{
	// file selected
	if($F(this.prefix + "_File").length == 0)
	{
		return;
	}

	// don't upload while another upload is taking place
	if (this.isUploading)
	{
		return;
	}
	
	// store key
	this.request.cmd  = "uploadGetProgress";
	this.request.pKey = $F(this.prefix + "_pKey");

	// submit form, i.e., send the file
	$(this.prefix + "_Form").submit();
	// FIXME makes FF not send data for regular inputs
	//$(this.prefix + "_Form").disable(); 
	
	// disable form
	this.isUploading = true;
	
	// start periodic upload progress checker
	THIS = this;
	this.poller = new PeriodicalExecuter(function(pe) { THIS.pollProgress(pe); }, this.pollInterval);
}

StudIPort.Upload.prototype.pollProgress = function(pe)
{
	//StudIPort.ajax.request(this.request, this.pollProgressSuccess, this.pollProgressFailure)
	// FIXME this is a very ugly workaround -> fix it appropriately!
	THIS = this;
	StudIPort.ajax.request(this.request, function(res, par) { THIS.pollProgressSuccess(res, par); }, function(res, par) { this.pollProgressFailure(res, par);  })
}

StudIPort.Upload.prototype.pollProgressSuccess = function(res, par)
{
	// check, if apc is installed
	if (res.noapc) {
		this.poller.stop();
		$(this.prefix + "_ProgressText").update("upload progress is not available");
		return;
	}

	// if there is a new pkey, update the form
	if (res.pKeyNext) {
		$(this.prefix + "_pKey").value = res.pKeyNext;
	}

	// display progress information
	progress = res.progress;
	// FIXME this version of "done" does not imply that we
	// really succeeded! Hence, postpone the feedback to
	// the final upload handling script!
	if (progress.done || progress.cancel) {
		this.poller.stop();
		if (progress.cancel) {
			// TODO show error message
			alert("Hochgeladen wurde abgebrochen");
		}
	} else {
		var perc = parseInt(progress.perc) + "%";
		var details = perc + " von " + progress.totalSize + " (" + progress.kps + " kB/s)";
		// TODO use progress.timeLeft
		$(this.prefix + "_ProgressBar").style.width = perc;
		$(this.prefix + "_ProgressText").update(details);
	}
}

StudIPort.Upload.prototype.pollProgressFailure = function(res, par)
{
	this.poller.stop();
	alert(res.errorMsg);
	return;
}

/**
 * invoked by StudIPortUpload.php to display a successful upload
 */
StudIPort.Upload.prototype.success = function(pkey, msg, usage, fileList)
{
	// show success message 
	$(this.prefix + "_ProgressBar").style.width = "100%";
	$(this.prefix + "_ProgressText").update("100% - Fertig");
	$(this.prefix + "_Msg").update(msg);

	// unlock the form
	this.unlock(pkey);

	// update usage statistics
	StudIPort.upload.updateUsageStat(usage);

	// update list of not associated files
	// TODO make this a function, as it is redundant code!
	//alert(msg + fileList.num + fileList.htmlContent);
	if (fileList.num > 0)
	{
		$("files").update(fileList.htmlContent);
	}
	else
	{
		$("files").update("");
	}
	
	Sortable.create("files", {constraint: "", overlap: "horizontal", ghosting: true});

	// clear iframe to prevent accidental re-uploading on page refresh
	$(this.prefix + "_iframe").src = "";
}

/* update the upload/usage statistics */
StudIPort.upload.updateUsageStat = function(usage)
{
	$("QuotaText").update(usage.curUsage);
	$("QuotaBar").style.width = usage.curUsagePerc + "%";
	$("MaxFileSize").update(usage.maxFileSizeHuman);
	// FIXME does not work with generic call of function
	//$(this.prefix + "_MaxFileSize").value = usage.maxFileSize;	
}

/**
 * invoked by StudIPortUpload.php to display an error message
 * @param msg error message to be displayed
 */
StudIPort.Upload.prototype.error = function(pkey, msg)
{
	$(this.prefix + "_ProgressText").update("Abgebrochen");
	$(this.prefix + "_Msg").update("<b>Fehler</b>: " + msg);
	
	this.unlock(pkey);
}

StudIPort.Upload.prototype.unlock = function(pkey)
{
	if (this.poller != null)
	{
		this.poller.stop();
	}
	
	this.isUploading = false;
	this.request     = {};
	
	$(this.prefix + "_File").setValue("");
	$(this.prefix + "_Form").enable();
	keyFld = $(this.prefix + "_pKey")
	if (keyFld) {
		keyFld.value = pkey;
	}
}

/**
 * reset the progress bar, invoked by file field's (onchange)
 */
StudIPort.Upload.prototype.reset = function()
{
	$(this.prefix + "_ProgressBar").style.width = "0%";
	$(this.prefix + "_ProgressText").update(" ");
	$(this.prefix + "_Msg").update(" ");
}
