<?php
/**************************************************************************
 * $Id: StudIPortFeedbackListExtAcc.class.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


require_once('StudIPortBase.class.php');
require_once('StudIPortTools.lib.php');
require_once('StudIPortFeedbackList.class.php');

/** @class StudIPortFeedbackList
 */
class StudIPortFeedbackListExtAcc extends StudIPortFeedbackList
{
	/** loads files associated with matrix
	 * @see StudIPortMatrixBase::loadData()
	 * @param void
	 */
	public function loadData() {
		// if a view is specified, get files of that view for the checkboxes
		if (empty($this->viewId) || !isset($_SESSION['StudIPortExtAcc']['access_code'])) {
			// ERROR
			return;
		}
		
		$sql =& $this->sql;
	
		// get questions associated with this view, which have a pending request or a reply from current user
		// TODO tell, if a question's edited date is younger than the latest reply of the user
		$sql->queryf(
			"SELECT DISTINCT " .
			"q.question_id," .
			"q.text," .
			"DATE_FORMAT(q.created, '%%d.%%m.%%Y %%k:%%i Uhr') AS created," .
			"DATE_FORMAT(q.edited,  '%%d.%%m.%%Y %%k:%%i Uhr') AS edited," .
			"DATE_FORMAT(q.expires, '%%e. %%b %%Y') AS expires," .
			"IF(q.expires,q.expires<CURDATE(),FALSE) AS expired " .
			"FROM studiport_feedback_question AS q " .
			"LEFT JOIN studiport_view         AS view USING(view_id) " .
			"WHERE view_id='%d' " .
			"AND   q.public='Y' " .
			"AND   view.public='Y' " .
			"AND   view.access_code='%s' " .
			"ORDER by q.edited DESC",
			$this->viewId, $_SESSION['StudIPortExtAcc']['access_code']);

		while ($sql->next_record()) {
			$sql->Record['text_abbr'] = textAbbreviate($sql->f('text'), QUESTION_ABBR_LENGTH);
			
			$key = count($this->questions);
			$this->questions[$key] = $sql->Record;
			$this->questions[$key]['requestPending'] = true;
			
			$this->responses[$sql->f('question_id')] = array();
			
			if (! $sql->f('expired')) {
				$this->numRequests++;
			}
		}
		
		$sql->queryf(
			  " SELECT"
			. "   r.response_id,"
			. "   r.question_id,"
			. "   r.author,"
			. "   r.text,"
			. "   DATE_FORMAT(r.created, '%%d.%%m.%%Y %%k:%%i Uhr') AS created, "
			. "   DATE_FORMAT(r.edited,  '%%d.%%m.%%Y %%k:%%i Uhr') AS edited, "
			. "   r.viewed"
			. " FROM studiport_feedback_response AS r"
			. "   LEFT JOIN studiport_feedback_question AS q    USING(question_id)"
			. "   LEFT JOIN studiport_view              AS view USING(view_id)"
			. " WHERE view_id='%d'"
			. "   AND view.access_code='%s'"
			. "   AND view.public='Y'"
			. "   AND q.public='Y'"
			. "   AND NOT r.author='NULL'"
			. " ORDER by r.edited DESC",
			$this->viewId, $_SESSION['StudIPortExtAcc']['access_code']);

		while ($sql->next_record()) {
			$sql->Record['text_abbr'] = textAbbreviate($sql->f('text'), RESPONSE_ABBR_LENGTH);
			array_push($this->responses[$sql->f('question_id')], $sql->Record);
		}
		
	}
	
	public function render(Flexi_TemplateFactory &$factory, $templateFilename, $pluginUrl) {
		parent::render($factory, $templateFilename, $pluginUrl);
		$this->template->set_attribute('isExtern',  true);
		return $this->template->render();
	}
}
?>
