<?php
/**************************************************************************
 * $Id: StudIPortFileTools.lib.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


$base = dirname(__FILE__) . "/../";
require_once($base . "config/StudIPort.config.php");
require_once($base . "lib/StudIPortDB.class.php");
require_once($base . "lib/PluginLink.lib.php");
require_once($base . "lib/StudIPortUploadTools.lib.php");
//require_once($base . 'lib/log_events.inc.php');
require_once('vendor/flexi/flexi.php');


function fileGetDetails($fileId) {
	$db =& new DB_Seminar();

	// sanitize id
	$id_esc = mysql_real_escape_string($fileId);
	$db->query("SELECT file_id, filename, DATE_FORMAT(created, '%d.%m.%y, %k:%i Uhr') as created, DATE_FORMAT(edited, '%d.%m.%y, %k:%i Uhr') as edited, title, descr_int, descr_ext, mimetype, type.name as filetype, filesize, type.picture FROM
studiport_file AS file LEFT JOIN studiport_file_types AS type USING (type_id) WHERE file.file_id='$id_esc'");

	if ($db->num_rows()) {
		$db->next_record();
		$res =& $db->Record;
		/*$fields = array('file_id', 'picture', 'title', 'descr_int', 'descr_ext', 'filename', 'created', 'edited', 'f
iletype');
		foreach ($fields as $col) {
			$res[$col] = $res[$col];
		}*/
		// convert filesize to human readable string
		$res['filesize'] = uploadSizeToString($res['filesize'], 1);

		return $res;
	} else {
		return null;
	}
}


/** deletes a file from studiport
 * @param fileIds of file to delete (comma separated string!)
 */
function fileDelete($fileIds) {
	global $user;

	if (empty($fileIds)) {
		return false;
	}
	
	$fileIds  = explode(",", $fileIds);
	$numFiles = count($fileIds);
	
	// check if fileIDs are of type numeric
	if (!array_isNumeric($fileIds)) {
		return false;
	}
	
	$fileIdString = implode(",", $fileIds);
	
	// check, if user requesting the deletion is owner
	for ($i = 0; $i < $numFiles; $i++) {
		if (!fileIsOwner($fileIds[$i], $user->id)) {
			return false;
		}
	}
	
	// get database connection
	$sql =& new StudIPortDB();
	
	$sql->queryf("SELECT int_filename FROM studiport_file WHERE file_id IN(%s)", $fileIdString);
	
	while ($sql->next_record()) {
		$filenames[] = $sql->Record["int_filename"];
	}
	
	// tables where data needs to be deleted
	$tables = array(
		array("name" => "file_assoc", "check" => false),
		array("name" => "view_files", "check" => false),
		array("name" => "file",       "check" => true),
	);
	
	$success = true;
	
	// perform query
	foreach ($tables as &$value) {
		$sql->queryf("DELETE FROM studiport_%s WHERE file_id IN(%s)", $value["name"], $fileIdString);
		
		if($value['check'] && $sql->affected_rows() != $numFiles) {
			$success = false;
		}
	}
	
	if ($success) {
		foreach ($filenames as $filename) {
			if (is_null($filename)) {
				continue;
			}

			$path = $GLOBALS['STUDIPORT_DOWNLOAD_DIR'] . '/' . $filename;
			
			// physical delete file
			// we dont need is_file here because errors are suppressed by @
			if(!@unlink($path)) {
				// log erros to studip loggin system
				log_event("STUDIPORT_DEL_FILE_ERR", $user_id_esc, NULL, $path, implode(",", error_get_last()));
				debuglog("filesystem", $user_id_esc, "Konnte Datei nicht aus FS loeschen", error_get_last());
			}
		}
		
		return true;
	} else {
		return false;
	}
}

/** deletes the association of a given file
 * @param fileId of the file
 */
function fileRelease($fileId) {
	if (empty($fileId) || !fileIsOwner($fileId, $GLOBALS['user']->id)) {
		return false;
	}
	
	// get database connection
	$sql =& new DB_Seminar();
	
	// clean up
	$file_esc = mysql_real_escape_string(intval($fileId));
	$user_id_esc = mysql_real_escape_string($GLOBALS['user']->id);
	
	// actually delete data from DB and verify success
	$sql->queryf("DELETE FROM studiport_file_assoc WHERE file_id = %u", $file_esc);
	
	if ($sql->affected_rows() == 0) {
		return false;
	} else {
		return true;
	}
}

/** associates a given file to comp. and exp.
 * @param fileId of the file
 * @param compId
 * @param expId
 */
function fileAssoc($fileId, $compId, $expId) {
	if (empty($fileId) || empty($compId) || empty($expId) || !fileIsOwner($fileId, $GLOBALS['user']->id)) {
		return false;
	}
	
	// get database connection
	$sql =& new DB_Seminar();
	
	// clean up
	$file_esc = mysql_real_escape_string(intval($fileId));
	$comp_esc = mysql_real_escape_string(intval($compId));
	$exp_esc = mysql_real_escape_string(intval($expId));
	$user_id_esc = mysql_real_escape_string($GLOBALS['user']->id);
	
	// check if file already assigned
	$sql->queryf("SELECT COUNT(*) as num
				FROM studiport_file_assoc
				WHERE file_id = %u", $file_esc);
				
	$sql->next_record();
	
	if ($sql->Record["num"] != 0) {
		return false;
	}
	
	// values for fields
	$fields = array(
		'file_id' => $file_esc,
		'comp_id' => $comp_esc,
		'exp_id'  => $exp_esc
	);
	
	$setClauses = "";
	foreach ($fields as $key => &$value) {
		$setClauses[] = $key ."=". $value;
	}
	$fieldsStr = join(", ", $setClauses);

	// actually insert data into DB and verify success
	$sql->queryf("INSERT INTO studiport_file_assoc SET %s", $fieldsStr);
	
	if ($sql->affected_rows() == 0) {
		return false;
	} else {
		return true;
	}
}

/** returns association keys of the given file
 * @param fileId
 * @return assoc. array with file_id, type_id, comp_id, exp_id
 */
function fileGetAssoc($fileId) {
	$res = array();
		
	if (empty($fileId) || !fileIsOwner($fileId, $GLOBALS['user']->id)) {
		return null;
	}
	
	// get database connection
	$sql =& new DB_Seminar();
	
	// clean up
	$file_esc    = mysql_real_escape_string(intval($fileId));
	$user_id_esc = mysql_real_escape_string($GLOBALS['user']->id);
	
	$sql->queryf("SELECT file_id, type_id, comp_id, exp_id FROM studiport_file LEFT JOIN studiport_file_assoc USING (file_id) WHERE file_id = %u", $file_esc);
	
	$sql->next_record();
	
	foreach($sql->Record as $key => &$value) {
		if(is_numeric($key)) continue;
		
		$res[$key] = $value;
	}
	
	return $res;
}

/** checks whether file belongs to user
 * @param fileId of the file
 * @param userId of the user
 */
function fileIsOwner($fileId, $userId) {
	if (empty($fileId) || empty($userId)) {
		return false;
	}
	
	// get database connection
	$sql =& new DB_Seminar();
	
	// clean up
	$file_esc = mysql_real_escape_string(intval($fileId));
	$user_id_esc = mysql_real_escape_string($userId);
		
	// check if file belongs to user
	$sql->queryf("SELECT COUNT(*) as num
				FROM studiport_file
				WHERE file_id = %u
				AND user_id = '%s'", $file_esc, $user_id_esc);	
				
	$sql->next_record();

	if ($sql->Record["num"] == 0) {
		return false;
	} else {
		return true;
	}
}

/** returns html of not allocated files
 * @note user_id is now taken from $globals["user"]
 */
function filesGetNoAssocList($url = false) {
	$res = array(
			"number" => 0,
			"htmlContent" => "");
	$data = array();

	// get database connection
	$sql =& new DB_Seminar();
	
	// template stuff
	$factory = new Flexi_TemplateFactory($GLOBALS['STUDIPORT_BASEDIR'] ."/templates");
	$template = $factory->open("StudIPortMatrixListEl.php");
	
	$template->set_attribute("url", $url ? $url : $GLOBALS['STUDIPORT_URI']);

	// clean up
	$sql->queryf("SELECT file_id, type_id, picture, title FROM studiport_file
					LEFT JOIN studiport_file_assoc USING (file_id)
					LEFT JOIN studiport_file_types types USING (type_id)
					WHERE user_id = '%s'
					AND comp_id IS NULL
					AND exp_id IS NULL", $GLOBALS['user']->id);

	while ($sql->next_record()) {
			$sql->Record["link"] = getPluginLink('artefacts', array('file_id' => $sql->Record['file_id']));
			$types[$sql->Record["type_id"]]["picture"] = $sql->Record["picture"]; // temp workaround
			$data[$sql->Record["file_id"]] = $sql->Record;
	}
	
	if ($sql->num_rows() == 0) {
		$template->set_attribute("files", array());
	} else {
		$template->set_attribute("files", $data);
		$template->set_attribute("types", $types);
	}

	// prepare result
	$res["number"]		= $sql->num_rows();
	$res["htmlContent"] = $template->render();

	return $res;
}

?>
