<?php
/**************************************************************************
 * $Id: StudIPortTools.lib.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


class MySQLSetClause {
	/* members */
	private $values = array();

	/* methods */
	public function __construct () {}

	public function set($field, $value, $quote = true) {
		//$value_esc = mysql_real_escape_string(utf8_decode($value));
		$value_esc = mysql_real_escape_string($value);
		if ($quote) {
			$value_esc = "'". $value_esc . "'";
		}
		$this->values[$field] = $value_esc;
	}

	public function get($forSprintf = false) {
		$v = array();
		foreach ($this->values as $key => &$value) {
			// escape % for usage with sprintf
			if ($forSprintf) {
				$value = str_replace("%", "%%", $value);
			}
			$v[] = $key ."=". $value;
		}
		return implode(', ', $v);
	}
	
	public function count() {
		return count($this->values);
	}
}


function setErrorMsg(array &$res, $msg) {
	$res['success']  = false;
	$res['errorMsg'] = "$msg";
}


function addWarning(array &$res, $msg) {
	if($msg == "") return;
	
	if (! is_array($res['warnings'])) {
		$res['warnings'] = array();
	}
	array_push($res['warnings'], "$msg");
}


function addFormError(array &$res, $field, $msg) {
	if ($msg == "") {
		return;
	}
	
	$res['formErrors']["$field"] = "$msg";
}


function array_utf8_encode(array &$arr) {
	foreach ($arr as $key=>&$value) {
		$arr[$key] = utf8_encode($value);
	}
}


function sanitizeInput ($val) {
        return utf8_decode(strip_tags(trim(undoMagicQuotes($val))));
}

// removes slashes for magic quoted strings (variables from gpc containing special chars, such as ',",\)
function undoMagicQuotes($str) {
	return get_magic_quotes_gpc() ? stripSlashes($str) : $str;
}

function array_isNumeric($array) {
	for ($i = 0; $i < count($array); $i++) {
		if (!is_numeric($array[$i]))
			return false;
	}
	
	return true;
}

function remove_dir($current_dir)
{
	if($dir = @opendir($current_dir)) {
		while (($f = readdir($dir)) !== false) {
			if($f > '0' and filetype($current_dir . '/' . $f) == "file") {
				unlink($current_dir . '/' . $f);
			} elseif($f > '0' and filetype($current_dir . '/' . $f) == "dir") {
				remove_dir($current_dir . '/' . $f."\\");
			}
		}
		closedir($dir);
		rmdir($current_dir);
	}
}

/**
 * truncates a string, if it has at least maxLen chars and appends '...'
 */
function textAbbreviate($str, $maxLen, $minLen = -1)
{
	// if string is shorter than allowed length, don't change it
	$str = rtrim($str);
	$len = strlen($str);
	if ($len < $maxLen - 3) {  // FIXME
		return $str;
	}

	// setup minLen, if not provided
	if (! is_int($minLen) || $minLen < 0) {
		$minLen = intval($maxLen / 2);
	}
	// if there is no space for the trailing dots, get it
	if ($minLen > $maxLen - 3) {
		$minLen = $maxLen - 3;
	}

	// remove three chars from the string for the dots
	$str = substr($str, 0, $maxLen - 3);

	// find last blank
	$subLen = strrpos($str, ' ');
	if ($subLen === false) {
		// there only is one word
		$subLen = $maxLen - 3;
	} else {
		$subLen += 1;  // convert index to length

		// if there is no blank or if it's the first char, move cursor to $maxLen-3
		// to simply replace the last three chars by dots
		if ($subLen < $minLen) {
			$subLen = $minLen;  // subtract 1 to compensate for following substr
		}
	}


	return substr($str, 0, $subLen) . '...';
}

function textConvertNewline($str)
{
	return preg_replace('/(\r?\n)+/', '<br />', $str);
}

?>
